@@ -13,7 +13,7 @@ These classes handle the detector-dependent paths and data filenames, on top of
 detector-dependent properties (e.g., number and size of the pixels, gaps between
 tiles...). Generic methods are implemented in the abstract base class Detector, and
 detector-dependent properties need to be implemented in each child class (they are
-decoracted by @abstractmethod in the base class, they are indicated using @ in the
+decoracted by @abstractmethod in the base class, they are written in italic in the
 following diagram).
 
 .. mermaid::
@@ -21,6 +21,7 @@ following diagram).
 
   classDiagram
     class Detector{
+      <<abstract>>
       +str name
       +tuple binning
       +str datadir
@@ -38,8 +39,8 @@ following diagram).
       +tuple sum_roi
       +str template_file
       +str template_imagefile
-      unbinned_pixel_number(@)
-      unbinned_pixel_size(@)
+      unbinned_pixel_number()*
+      unbinned_pixel_size()*
       _background_subtraction()
       _flatfield_correction()
       _hotpixels_correction()
@@ -57,23 +58,27 @@ API Reference
 
 """
 
-from abc import ABC, abstractmethod
-from functools import partial
-import numpy as np
-from numbers import Real, Integral
+import logging
 import os
 import pathlib
-from typing import Any, Dict, List, Union
+from abc import ABC, abstractmethod
+from functools import partial
+from numbers import Integral, Real
 
+import numpy as np
+
+from bcdi.utils import utilities as util
 from bcdi.utils import validation as valid
 
+module_logger = logging.getLogger(__name__)
+
 
 def create_detector(name, **kwargs):
     """
     Create a Detector instance depending on the detector.
 
     :param name: str, name of the detector
-    :return:  the corresponding diffractometer instance
+    :return:  the corresponding detector instance
     """
     if name == "Maxipix":
         return Maxipix(name=name, **kwargs)
@@ -81,70 +86,25 @@ def create_detector(name, **kwargs):
         return Eiger2M(name=name, **kwargs)
     if name == "Eiger4M":
         return Eiger4M(name=name, **kwargs)
+    if name == "Eiger9M":
+        return Eiger9M(name=name, **kwargs)
     if name == "Timepix":
         return Timepix(name=name, **kwargs)
     if name == "Merlin":
         return Merlin(name=name, **kwargs)
+    if name == "MerlinSixS":
+        return MerlinSixS(name=name, **kwargs)
     if name == "Dummy":
         return Dummy(name=name, **kwargs)
     raise NotImplementedError(f"No implementation for the {name} detector")
 
 
-def create_roi(dic: Dict[str, Any]) -> Any:
-    """
-    Load "roi_detector" from the dictionary of parameters and update it.
-
-    If the keys "center_roi_x" or "center_roi_y" are defined, it will consider that the
-    current values in roi_detector define a window around the Bragg peak position and
-    the final output will be:
-    [center_roi_y - roi_detector[0], center_roi_y + roi_detector[1],
-    center_roi_x - roi_detector[2], center_roi_x + roi_detector[3]].
-
-    If a key is not defined, it will consider that the values of roi_detector are
-    absolute pixels positions, e.g. if only "center_roi_y" is defined, the output will
-    be:
-    [center_roi_y - roi_detector[0], center_roi_y + roi_detector[1],
-    roi_detector[2], roi_detector[3]].
-
-    Accordingly, if none of the keys are defined, the output will be:
-    [roi_detector[0], roi_detector[1], roi_detector[2], roi_detector[3]].
-
-    :param dic: a dictionary of parameters
-    :return: the calculated region of interest [Vstart, Vstop, Hstart, Hstop] or None
-    """
-    valid.valid_container(dic, container_types=dict, name="dic")
-    roi = dic.get("roi_detector")
-    valid.valid_container(
-        roi,
-        container_types=(tuple, list, np.ndarray),
-        length=4,
-        item_types=int,
-        allow_none=True,
-        name="roi_detector",
-    )
-
-    # update the ROI
-    if roi is not None:
-        center_roi_y = dic.get("center_roi_y")
-        if center_roi_y is not None:
-            valid.valid_item(center_roi_y, allowed_types=int, name="center_roi_y")
-            roi[0] = center_roi_y - roi[0]
-            roi[1] = center_roi_y + roi[1]
-
-        center_roi_x = dic.get("center_roi_x")
-        if center_roi_x is not None:
-            valid.valid_item(center_roi_x, allowed_types=int, name="center_roi_x")
-            roi[2] = center_roi_x - roi[2]
-            roi[3] = center_roi_x + roi[3]
-    return roi
-
-
 class Detector(ABC):
     """
     Class to handle the configuration of the detector used for data acquisition.
 
-    :param name: name of the detector in {'Maxipix', 'Timepix', 'Merlin', 'Eiger2M',
-     'Eiger4M', 'Dummy'}
+    :param name: name of the detector in {'Maxipix', 'Timepix', 'Merlin',
+     'MerlinSixS', 'Eiger2M', 'Eiger4M', 'Dummy'}
     :param datadir: directory where the data files are located
     :param savedir: directory where to save the results
     :param template_imagefile: beamline-dependent template for the data files
@@ -172,6 +132,7 @@ class Detector(ABC):
        parameter delta in xrayutilities hxrd.Ang2Q.area method
      - 'linearity_func': function to apply to each pixel of the detector in order to
        compensate the deviation of the detector linearity for large intensities.
+     - 'logger': an optional logger
 
     """
 
@@ -195,6 +156,7 @@ class Detector(ABC):
         self._name = name
 
         # load the kwargs
+        self.logger = kwargs.get("logger", module_logger)
         self.preprocessing_binning = kwargs.get("preprocessing_binning") or (1, 1, 1)
         self.offsets = kwargs.get("offsets")  # delegate the test to xrayutilities
         self.linearity_func = kwargs.get("linearity_func")
@@ -219,6 +181,11 @@ class Detector(ABC):
         # initialize the threshold for saturation, can be overriden in child classes
         self.saturation_threshold = None
 
+        # property used to track the binning factor throughout data processing
+        # the starting point is preprocessing_binning, which is the state of the data
+        # when loaded
+        self.current_binning = list(self.preprocessing_binning)
+
     @property
     def binning(self):
         """
@@ -253,6 +220,28 @@ class Detector(ABC):
             raise TypeError("beamline should be a string")
         return self._counter_table.get(beamline)
 
+    @property
+    def current_binning(self):
+        """
+        Display the current binning factor of the dataset.
+
+        Tuple of three positive integers corresponding to the current binning of the
+        data in the processing pipeline.
+        """
+        return self._current_binning
+
+    @current_binning.setter
+    def current_binning(self, value):
+        valid.valid_container(
+            value,
+            container_types=list,
+            length=3,
+            item_types=int,
+            min_excluded=0,
+            name="Detector.current_binning",
+        )
+        self._current_binning = value
+
     @property
     def datadir(self):
         """Name of the data directory."""
@@ -288,9 +277,9 @@ class Detector(ABC):
             :return: the updated 1D array
             """
             return (
-                coeffs[0] * array_1d ** 4
-                + coeffs[1] * array_1d ** 3
-                + coeffs[2] * array_1d ** 2
+                coeffs[0] * array_1d**4
+                + coeffs[1] * array_1d**3
+                + coeffs[2] * array_1d**2
                 + coeffs[3] * array_1d
                 + coeffs[4]
             )
@@ -449,7 +438,6 @@ class Detector(ABC):
         valid.valid_container(
             value,
             container_types=str,
-            min_length=1,
             allow_none=True,
             name="Detector.sample_name",
         )
@@ -479,8 +467,7 @@ class Detector(ABC):
         if self.datadir:
             dir_path = os.path.abspath(os.path.join(self.datadir, os.pardir)) + "/"
             return dir_path.replace("\\", "/")
-        else:
-            return None
+        return None
 
     @property
     def sum_roi(self):
@@ -556,24 +543,7 @@ class Detector(ABC):
 
     def __repr__(self):
         """Representation string of the Detector instance."""
-        return (
-            f"{self.__class__.__name__}(name='{self.name}', "
-            f"unbinned_pixel_size={self.unbinned_pixel_size}, "
-            f"nb_pixel_x={self.nb_pixel_x}, "
-            f"nb_pixel_y={self.nb_pixel_y}, "
-            f"binning={self.binning},\n"
-            f"roi={self.roi}, "
-            f"sum_roi={self.sum_roi}, "
-            f"preprocessing_binning={self.preprocessing_binning}, "
-            f"rootdir = {self.rootdir},\n"
-            f"datadir = {self.datadir},\n"
-            f"scandir = {self.scandir},\n"
-            f"savedir = {self.savedir},\n"
-            f"sample_name = {self.sample_name},"
-            f" template_file = {self.template_file}, "
-            f"template_imagefile = {self.template_imagefile},"
-            f" specfile = {self.specfile},\n"
-        )
+        return util.create_repr(self, Detector)
 
     @staticmethod
     def _background_subtraction(data, background):
@@ -778,7 +748,7 @@ class Eiger2M(Detector):
         super().__init__(name=name, **kwargs)
         self._counter_table = {"ID01": "ei2minr"}  # useful if the same type of detector
         # is used at several beamlines
-        self.saturation_threshold = 1e6
+        self.saturation_threshold = 1e7
 
     def _mask_gaps(self, data, mask):
         """
@@ -899,6 +869,74 @@ class Eiger4M(Detector):
         return 75e-06, 75e-06
 
 
+class Eiger9M(Detector):
+    """Implementation of the Eiger9M detector."""
+
+    def __init__(self, name, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self.saturation_threshold = 4000000000
+
+    def _mask_gaps(self, data, mask):
+        """
+        Mask the gaps between sensors in the detector.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        valid.valid_ndarray(
+            (data, mask), ndim=2, shape=self.unbinned_pixel_number, fix_shape=True
+        )
+        data[:, 0:1] = 0
+        data[:, -1:] = 0
+        data[0:1, :] = 0
+        data[-1:, :] = 0
+        data[:, 513:515] = 0
+        data[:, 1028:1040] = 0
+        data[:, 1553:1555] = 0
+        data[:, 2068:2080] = 0
+        data[:, 2593:2595] = 0
+        data[512:550, :] = 0
+        data[1062:1100, :] = 0
+        data[1612:1650, :] = 0
+        data[2162:2200, :] = 0
+        data[2712:2750, :] = 0
+
+        mask[:, 0:1] = 1
+        mask[:, -1:] = 1
+        mask[0:1, :] = 1
+        mask[-1:, :] = 1
+        mask[:, 513:515] = 1
+        mask[:, 1028:1040] = 1
+        mask[:, 1553:1555] = 1
+        mask[:, 2068:2080] = 1
+        mask[:, 2593:2595] = 1
+        mask[512:550, :] = 1
+        mask[1062:1100, :] = 1
+        mask[1612:1650, :] = 1
+        mask[2162:2200, :] = 1
+        mask[2712:2750, :] = 1
+        return data, mask
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        return 3262, 3108
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 75e-06, 75e-06
+
+
 class Timepix(Detector):
     """Implementation of the Timepix detector."""
 
@@ -965,6 +1003,51 @@ class Merlin(Detector):
         return 55e-06, 55e-06
 
 
+class MerlinSixS(Detector):
+    """Implementation of the Merlin detector for SixS."""
+
+    def __init__(self, name, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self.saturation_threshold = 1e6
+
+    def _mask_gaps(self, data, mask):
+        """
+        Mask the gaps between sensors in the detector.
+
+        :param data: a 2D numpy array
+        :param mask: a 2D numpy array of the same shape as data
+        :return:
+
+         - the masked data
+         - the updated mask
+
+        """
+        valid.valid_ndarray(
+            (data, mask), ndim=2, shape=self.unbinned_pixel_number, fix_shape=True
+        )
+
+        data[:, 254:257] = 0
+        data[254:257, :] = 0
+
+        mask[:, 254:257] = 1
+        mask[254:257, :] = 1
+        return data, mask
+
+    @property
+    def unbinned_pixel_number(self):
+        """
+        Define the number of pixels of the unbinned detector.
+
+        Convention: (vertical, horizontal)
+        """
+        return 512, 512
+
+    @property
+    def unbinned_pixel_size(self):
+        """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 55e-06, 55e-06
+
+
 class Dummy(Detector):
     """
     Implementation of the Dummy detector.
@@ -1011,7 +1094,7 @@ class Dummy(Detector):
             val is not None for val in self.custom_pixelnumber
         ):
             return self.custom_pixelnumber
-        print(f"Defaulting the pixel number to {516, 516}")
+        self.logger.info(f"Defaulting the pixel number to {516, 516}")
         return 516, 516
 
     @property
@@ -1019,5 +1102,5 @@ class Dummy(Detector):
         """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
         if self.custom_pixelsize is not None:
             return self.custom_pixelsize, self.custom_pixelsize
-        print(f"Defaulting the pixel size to {55e-06, 55e-06}")
+        self.logger.info(f"Defaulting the pixel size to {55e-06, 55e-06}")
         return 55e-06, 55e-06
