@@ -10,17 +10,19 @@
 
 """Postprocessing of the output from the facet analyzer plugin for Paraview."""
 
+import pathlib
+from typing import Any, Dict, List, Optional, Tuple, Union, no_type_check
+
 import h5py
 import ipywidgets as widgets
-from ipywidgets import Layout, interactive
 import matplotlib.pyplot as plt
 import numpy as np
-import pathlib
 import pandas as pd
-from pandas import DataFrame
-from typing import Any, Dict, List, no_type_check, Optional, Tuple, Union
 import vtk
+from ipywidgets import Layout, interactive
+from pandas import DataFrame
 
+from bcdi.utils import utilities as util
 from bcdi.utils import validation as valid
 
 
@@ -28,10 +30,11 @@ class Facets:
     """
     Import and stores data output of facet analyzer plugin for further analysis.
 
-    Extract the strain component and the displacement on the facets, and retrieves the
-    correct facet normals based on a user input (geometric transformation into the
-    crystal frame). It requries as input a VTK file extracted from the FacetAnalyser
-    plugin from ParaView. See: https://doi.org/10.1016/j.ultramic.2012.07.024
+    Extract the mean and standard deviation of the strain and displacement distribution
+    on the facets. Retrieves the correct facet normals based on a user input (geometric
+    transformation into the crystal frame). It requires as input a VTK file extracted
+    from the FacetAnalyser plugin from ParaView.
+    See: https://doi.org/10.1016/j.ultramic.2012.07.024
 
     Original tutorial on how to open vtk files:
     http://forrestbao.blogspot.com/2011/12/reading-vtk-files-in-python-via-python.html
@@ -81,9 +84,6 @@ class Facets:
         self.theoretical_angles: Optional[Dict[str, float]] = None
 
         # Check input parameters
-        valid.valid_container(
-            filename, container_types=str, min_length=1, name="filename"
-        )
         valid.valid_container(
             pathdir, container_types=str, min_length=1, name="pathdir"
         )
@@ -95,10 +95,12 @@ class Facets:
         if savedir is not None and not savedir.endswith("/"):
             savedir += "/"
         valid.valid_item(lattice, allowed_types=float, min_excluded=0, name="lattice")
+        self.filename = filename
         self.pathsave = savedir or pathdir + "facets_analysis/"
         self.path_to_data = pathdir + filename
-        self.filename = filename
         self.lattice = lattice
+        self.pathdir = pathdir
+        self.savedir = savedir
 
         # Plotting options
         self.strain_range = 0.001
@@ -175,6 +177,16 @@ class Facets:
             ),
         )
 
+    @property
+    def filename(self):
+        """Path to the VTK file."""
+        return self._filename
+
+    @filename.setter
+    def filename(self, value):
+        valid.valid_container(value, container_types=str, min_length=1, name="filename")
+        self._filename = value
+
     def load_vtk(self) -> None:
         """
         Load the VTK file.
@@ -1315,8 +1327,8 @@ class Facets:
                     }
                 )
 
-                self.field_data = self.field_data.append(
-                    edges_cornes_df, ignore_index=True
+                self.field_data = pd.concat(
+                    [self.field_data, edges_cornes_df], ignore_index=True
                 )
                 self.field_data = self.field_data.sort_values(by="facet_id")
                 self.field_data = self.field_data.reset_index(drop=True)
@@ -1391,19 +1403,36 @@ class Facets:
                 print("Saved Facets class attributes")
 
             except (AttributeError, TypeError):
-                print("Particle not rotated, some attributes could not be saved ...")
-                facets.create_dataset("u0", data=np.zeros(3))
-                facets.create_dataset("v0", data=np.zeros(3))
-                facets.create_dataset("w0", data=np.zeros(3))
-                facets.create_dataset("u", data=np.zeros(3))
-                facets.create_dataset("v", data=np.zeros(3))
-                facets.create_dataset("norm_u", data=np.zeros(3))
-                facets.create_dataset("norm_v", data=np.zeros(3))
-                facets.create_dataset("norm_w", data=np.zeros(3))
-                facets.create_dataset("rotation_matrix", data=np.zeros((3, 3)))
-                facets.create_dataset("hkl_reference", data=[])
-                facets.create_dataset("planar_dist", data="")
-                facets.create_dataset("ref_normal", data=np.zeros(3))
+                try:
+                    print(
+                        "Particle not rotated, some attributes could not be saved ..."
+                    )
+                    facets.create_dataset("u0", data=np.zeros(3))
+                    facets.create_dataset("v0", data=np.zeros(3))
+                    facets.create_dataset("w0", data=np.zeros(3))
+                    facets.create_dataset("u", data=np.zeros(3))
+                    facets.create_dataset("v", data=np.zeros(3))
+                    facets.create_dataset("norm_u", data=np.zeros(3))
+                    facets.create_dataset("norm_v", data=np.zeros(3))
+                    facets.create_dataset("norm_w", data=np.zeros(3))
+                    facets.create_dataset("rotation_matrix", data=np.zeros((3, 3)))
+                    facets.create_dataset("hkl_reference", data=[])
+                    facets.create_dataset("planar_dist", data="")
+                    facets.create_dataset("ref_normal", data=np.zeros(3))
+                except ValueError:
+                    f["/entry_1/process_4/u0"][...] = np.zeros(3)
+                    f["/entry_1/process_4/v0"][...] = np.zeros(3)
+                    f["/entry_1/process_4/w0"][...] = np.zeros(3)
+                    f["/entry_1/process_4/u"][...] = np.zeros(3)
+                    f["/entry_1/process_4/v"][...] = np.zeros(3)
+                    f["/entry_1/process_4/norm_u"][...] = np.zeros(3)
+                    f["/entry_1/process_4/norm_v"][...] = np.zeros(3)
+                    f["/entry_1/process_4/norm_w"][...] = np.zeros(3)
+                    f["/entry_1/process_4/rotation_matrix"][...] = np.zeros((3, 3))
+                    f["/entry_1/process_4/hkl_reference"][...] = []
+                    f["/entry_1/process_4/planar_dist"][...] = ""
+                    f["/entry_1/process_4/planar_dist"].attrs["units"] = "Angstrom"
+                    f["/entry_1/process_4/ref_normal"][...] = np.zeros(3)
 
         # Save field data
         try:
@@ -1447,9 +1476,7 @@ class Facets:
 
     def __repr__(self):
         """Unambiguous representation of the class."""
-        return "Facets {}\n".format(
-            self.filename,
-        )
+        return util.create_repr(obj=self, cls=Facets)
 
     def __str__(self):
         """Readable representation of the class."""
