@@ -8,13 +8,15 @@
 """Functions related to data postprocessing after phase retrieval."""
 
 import gc
+import logging
 from math import pi
 from numbers import Number, Real
+
+import matplotlib.pyplot as plt
 import numpy as np
 import numpy.ma as ma
-from numpy.fft import fftn, fftshift, ifftn, ifftshift
 import scipy
-import matplotlib.pyplot as plt
+from numpy.fft import fftn, fftshift, ifftn, ifftshift
 from scipy.interpolate import RegularGridInterpolator
 from scipy.ndimage.measurements import center_of_mass
 from scipy.signal import convolve
@@ -25,6 +27,8 @@ from ..graph import graph_utils as gu
 from ..utils import utilities as util
 from ..utils import validation as valid
 
+module_logger = logging.getLogger(__name__)
+
 
 def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
     """
@@ -37,6 +41,8 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
     :param debugging: set to True to see plots
     :type debugging: bool
     :param kwargs:
+
+     - 'cmap': str, name of the colormap
      - for the normal distribution: 'sigma' and 'mu' of the 3d multivariate normal
        distribution, tuples of 3 floats
      - for the Tuckey window: 'alpha' (shape parameter) of the 3d Tukey window,
@@ -50,14 +56,14 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
     # check and load kwargs
     valid.valid_kwargs(
         kwargs=kwargs,
-        allowed_kwargs={"sigma", "mu", "alpha", "is_orthogonal"},
+        allowed_kwargs={"cmap", "sigma", "mu", "alpha", "is_orthogonal"},
         name="postprocessing_utils.apodize",
     )
     sigma = kwargs.get("sigma")
     mu = kwargs.get("mu")
     alpha = kwargs.get("alpha")
     is_orthogonal = kwargs.get("is_orthogonal", False)
-
+    cmap = kwargs.get("cmap", "turbo")
     # calculate the diffraction pattern of the reconstructed object
     nb_z, nb_y, nb_x = amp.shape
     nbz, nby, nbx = initial_shape
@@ -73,6 +79,7 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
             reciprocal_space=False,
             is_orthogonal=is_orthogonal,
             scale="linear",
+            cmap=cmap,
         )
 
     my_fft = fftshift(fftn(myobj))
@@ -89,6 +96,7 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
             reciprocal_space=True,
             is_orthogonal=is_orthogonal,
             scale="log",
+            cmap=cmap,
         )
 
     if window_type == "normal":
@@ -102,7 +110,7 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
             np.linspace(-1, 1, nbx),
             indexing="ij",
         )
-        covariance = np.diag(sigma ** 2)
+        covariance = np.diag(sigma**2)
         window = multivariate_normal.pdf(
             np.column_stack([grid_z.flat, grid_y.flat, grid_x.flat]),
             mean=mu,
@@ -138,6 +146,7 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
             reciprocal_space=True,
             is_orthogonal=is_orthogonal,
             scale="log",
+            cmap=cmap,
         )
 
     myobj = ifftn(ifftshift(my_fft))
@@ -152,6 +161,7 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
             reciprocal_space=False,
             is_orthogonal=is_orthogonal,
             scale="linear",
+            cmap=cmap,
         )
     myobj = util.crop_pad(
         myobj, (nb_z, nb_y, nb_x)
@@ -249,29 +259,22 @@ def bragg_temperature(
         reflection
     )  # go back to lattice constant
     print(
-        "Reference spacing at",
-        temperature_ref,
-        "K   =",
-        str("{:.4f}".format(spacing_ref)),
-        "angstroms",
-    )
-    print(
-        "Spacing =",
-        str("{:.4f}".format(spacing)),
-        "angstroms using reflection",
-        reflection,
+        f"Reference spacing at {temperature_ref} K = {spacing_ref:.4f} angstroms",
     )
+    print(f"Spacing = {spacing:.4f} angstroms using reflection {reflection}")
 
-    # fit the experimental spacing with non corrected platinum curve
+    # fit the experimental spacing with non-corrected platinum curve
     myfit = np.poly1d(np.polyfit(expansion_data[:, 2], expansion_data[:, 0], 3))
-    print("Temperature without offset correction=", int(myfit(spacing) - 273.15), "C")
+    print(
+        f"Temperature without offset correction = {int(myfit(spacing) - 273.15)} degC"
+    )
 
     # find offset for platinum reference curve
     myfit = np.poly1d(np.polyfit(expansion_data[:, 0], expansion_data[:, 2], 3))
     spacing_offset = (
         myfit(temperature_ref) - spacing_ref
     )  # T in K, spacing in angstroms
-    print("Spacing offset =", str("{:.4f}".format(spacing_offset)), "angstroms")
+    print(f"Spacing offset = {spacing_offset:.4f} angstroms")
 
     # correct the platinum reference curve for the offset
     platinum_offset = np.copy(expansion_data)
@@ -335,18 +338,18 @@ def calc_coordination(
     return mycoord
 
 
-def center_com(array, debugging=False, **kwargs):
+def center_com(array: np.ndarray, debugging: bool = False, **kwargs) -> np.ndarray:
     """
     Center array based on center_of_mass(abs(array)) using pixel shift.
 
-    :param array: 3D array to be centered based on the center of mass of abs(array)
+    :param array: 3D array to be centered using the center of mass of abs(array)
     :param debugging: boolean, True to see plots
     :param kwargs:
-     - width_z: size of the area to plot in z (axis 0), centered on the middle
+     - width_z: size of the area to plot in z (axis 0), centered in the middle
        of the initial array
-     - width_y: size of the area to plot in y (axis 1), centered on the middle
+     - width_y: size of the area to plot in y (axis 1), centered in the middle
        of the initial array
-     - width_x: size of the area to plot in x (axis 2), centered on the middle
+     - width_x: size of the area to plot in x (axis 2), centered in the middle
        of the initial array
 
     :return: array centered by pixel shift
@@ -402,22 +405,10 @@ def center_com(array, debugging=False, **kwargs):
         )
 
         print(
-            "center of mass at (z, y, x): (",
-            str("{:.2f}".format(piz)),
-            ",",
-            str("{:.2f}".format(piy)),
-            ",",
-            str("{:.2f}".format(pix)),
-            ")",
+            f"center of mass at (z, y, x): ({piz:.2f},{piy:.2f},{pix:.2f})",
         )
         print(
-            "center of mass offset: (",
-            offset_z,
-            ",",
-            offset_y,
-            ",",
-            offset_x,
-            ") pixels",
+            f"center of mass offset: ({offset_z},{offset_y},{offset_x}) pixels",
         )
 
     #####################
@@ -436,18 +427,18 @@ def center_com(array, debugging=False, **kwargs):
     return array
 
 
-def center_max(array, debugging=False, **kwargs):
+def center_max(array: np.ndarray, debugging: bool = False, **kwargs) -> np.ndarray:
     """
     Center array based on max(abs(array)) using pixel shift.
 
     :param array: 3D array to be centered based on max(abs(array))
     :param debugging: boolean, True to see plots
     :param kwargs:
-     - width_z: size of the area to plot in z (axis 0), centered on the middle
+     - width_z: size of the area to plot in z (axis 0), centered in the middle
        of the initial array
-     - width_y: size of the area to plot in y (axis 1), centered on the middle
+     - width_y: size of the area to plot in y (axis 1), centered in the middle
        of the initial array
-     - width_x: size of the area to plot in x (axis 2), centered on the middle
+     - width_x: size of the area to plot in x (axis 2), centered in the middle
        of the initial array
 
     :return: array centered by pixel shift
@@ -521,6 +512,27 @@ def center_max(array, debugging=False, **kwargs):
     return array
 
 
+def center_object(method: str, obj: np.ndarray) -> np.ndarray:
+    """
+    Center an object based on its modulus.
+
+    :param method: method used for centering. 'com' (center of mass), 'max', 'max_com'
+     (max then com), 'skip'.
+    :param obj: a 3D numpy array
+    :return: the centered array of the same shape as obj
+    """
+    valid.valid_ndarray(obj, ndim=3)
+    if method == "max":
+        return center_max(obj)
+        # shift based on max value, use this if the object spans across the edge of the
+        # window before using the center of mass
+    if method == "com":
+        return center_com(obj)
+    if method == "max_com":
+        return center_com(center_max(obj))
+    return obj
+
+
 def filter_3d(
     array, filter_name="gaussian_highpass", kernel_length=21, debugging=False, **kwargs
 ):
@@ -571,6 +583,7 @@ def find_bulk(
     width_y=None,
     width_x=None,
     debugging=False,
+    **kwargs,
 ):
     """
     Isolate the inner part of the crystal from the non-physical surface.
@@ -587,9 +600,14 @@ def find_bulk(
      of the initial array
     :param debugging: set to True to see plots
     :type debugging: bool
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+
     :return: the support corresponding to the bulk
     """
     valid.valid_ndarray(arrays=amp, ndim=3)
+    cmap = kwargs.get("cmap", "turbo")
 
     nbz, nby, nbx = amp.shape
     max_amp = abs(amp).max()
@@ -622,6 +640,7 @@ def find_bulk(
                 vmin=0,
                 vmax=1,
                 title="Outer matrix",
+                cmap=cmap,
             )
 
         ############################################################################
@@ -659,6 +678,7 @@ def find_bulk(
                     vmin=0,
                     vmax=1,
                     title="Surface matrix",
+                    cmap=cmap,
                 )
 
             # second step: calculate the % of voxels from that layer whose amplitude
@@ -670,12 +690,9 @@ def find_bulk(
             )  # % of voxels whose amplitude is larger than support_threshold
             mean_amp = np.mean(amp[np.nonzero(surface)].flatten()) / max_amp
             print(
-                "number of surface voxels =",
-                nb_voxels,
-                "  , % of surface voxels above threshold =",
-                str("{:.2f}".format(100 * voxels_counter)),
-                "%    , mean surface amplitude =",
-                mean_amp,
+                f"number of surface voxels = {nb_voxels}, "
+                f"% of surface voxels above threshold = {100 * voxels_counter:.2f} %, ",
+                f"mean surface amplitude = {mean_amp}",
             )
             if mean_amp < support_threshold:
                 outer[np.nonzero(surface)] = 1
@@ -820,17 +837,21 @@ def find_datarange(array, plot_margin=10, amplitude_threshold=0.1, keep_size=Fal
     return zrange, yrange, xrange
 
 
-def flip_reconstruction(obj, debugging=False):
+def flip_reconstruction(obj, debugging=False, **kwargs):
     """
     Calculate the conjugate object giving the same diffracted intensity as 'obj'.
 
     :param obj: 3D reconstructed complex object
     :param debugging: set to True to see plots
     :type debugging: bool
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+
     :return: the flipped complex object
     """
     valid.valid_ndarray(arrays=obj, ndim=3)
-
+    cmap = kwargs.get("cmap", "turbo")
     flipped_obj = ifftn(ifftshift(np.conj(fftshift(fftn(obj)))))
     if debugging:
         gu.multislices_plot(
@@ -839,6 +860,7 @@ def flip_reconstruction(obj, debugging=False):
             sum_frames=False,
             plot_colorbar=True,
             title="Initial object",
+            cmap=cmap,
         )
         gu.multislices_plot(
             abs(flipped_obj),
@@ -846,6 +868,7 @@ def flip_reconstruction(obj, debugging=False):
             sum_frames=False,
             plot_colorbar=True,
             title="Flipped object",
+            cmap=cmap,
         )
     return flipped_obj
 
@@ -912,6 +935,8 @@ def get_opticalpath(support, direction, k, voxel_size=None, debugging=False, **k
     :param voxel_size: tuple, actual voxel size in z, y, and x (CXI convention)
     :param debugging: boolena, True to see plots
     :param kwargs:
+
+     - 'cmap': str, name of the colormap
      - width_z: size of the area to plot in z (axis 0), centered on the middle
        of the initial array
      - width_y: size of the area to plot in y (axis 1), centered on the middle
@@ -925,8 +950,11 @@ def get_opticalpath(support, direction, k, voxel_size=None, debugging=False, **k
     # check and load kwargs #
     #########################
     valid.valid_kwargs(
-        kwargs=kwargs, allowed_kwargs={"width_z", "width_y", "width_x"}, name="kwargs"
+        kwargs=kwargs,
+        allowed_kwargs={"cmap", "width_z", "width_y", "width_x"},
+        name="kwargs",
     )
+    cmap = kwargs.get("cmap", "turbo")
     width_z = kwargs.get("width_z")
     valid.valid_item(
         value=width_z,
@@ -1059,6 +1087,7 @@ def get_opticalpath(support, direction, k, voxel_size=None, debugging=False, **k
             title="Support for optical path",
             is_orthogonal=True,
             reciprocal_space=False,
+            cmap=cmap,
         )
 
     ###########################################
@@ -1071,6 +1100,7 @@ def get_opticalpath(support, direction, k, voxel_size=None, debugging=False, **k
         half_width=1,
         title="Optical path",
         debugging=debugging,
+        cmap=cmap,
     )
 
     return path
@@ -1084,6 +1114,7 @@ def get_strain(
     extent_phase=2 * pi,
     method="default",
     debugging=False,
+    **kwargs,
 ):
     """
     Calculate the 3D strain array.
@@ -1101,9 +1132,14 @@ def get_strain(
     :param method: 'default' or 'defect'. If 'defect', will offset the phase
      in a loop and keep the smallest value for the strain (Felix Hofmann's method 2019).
     :param debugging: True to see plots
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+
     :return: the strain 3D array
     """
     # check some parameters
+    cmap = kwargs.get("cmap", "turbo")
     valid.valid_ndarray(arrays=phase, ndim=3)
     if reference_axis not in {"x", "y", "z"}:
         raise ValueError("The reference axis should be 'x', 'y' or 'z'")
@@ -1166,6 +1202,7 @@ def get_strain(
                 plot_colorbar=True,
                 is_orthogonal=True,
                 reciprocal_space=False,
+                cmap=cmap,
             )
             gu.multislices_plot(
                 strain,
@@ -1176,6 +1213,7 @@ def get_strain(
                 plot_colorbar=True,
                 is_orthogonal=True,
                 reciprocal_space=False,
+                cmap=cmap,
             )
     return strain
 
@@ -1191,6 +1229,7 @@ def mean_filter(
     vmax=np.nan,
     title="Object",
     debugging=False,
+    **kwargs,
 ):
     """
     Apply a mean filter to an object defined by a support.
@@ -1212,11 +1251,16 @@ def mean_filter(
     :param vmax: real number, higher boundary for the colorbar of the plots
     :param title: str, title for the plots
     :param debugging: bool, True to see plots
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+
     :return: averaged array of the same shape as the input array
     """
     #########################
     # check some parameters #
     #########################
+    cmap = kwargs.get("cmap", "turbo")
     valid.valid_ndarray(arrays=(array, support), ndim=3)
     valid.valid_item(half_width, allowed_types=int, min_included=0, name="half_width")
     valid.valid_container(title, container_types=str, name="title")
@@ -1259,6 +1303,7 @@ def mean_filter(
                 vmax=vmax,
                 title=title + " before averaging",
                 plot_colorbar=True,
+                cmap=cmap,
             )
             gu.multislices_plot(
                 support,
@@ -1268,6 +1313,7 @@ def mean_filter(
                 vmin=0,
                 vmax=1,
                 title="Support for averaging",
+                cmap=cmap,
             )
 
         nonzero_pixels = np.argwhere(support != 0)
@@ -1309,6 +1355,7 @@ def mean_filter(
                 vmax=vmax,
                 title=title + " after averaging",
                 plot_colorbar=True,
+                cmap=cmap,
             )
         if counter != 0:
             print("There were", counter, "voxels for which phase could not be averaged")
@@ -1505,6 +1552,8 @@ def remove_offset(
     :param title: string, used in plot title
     :param debugging: True to see plots
     :param kwargs:
+
+     - 'cmap': str, name of the colormap
      - 'reciprocal_space': True if the object is in reciprocal space
      - 'is_orthogonal': True if the data is in an orthonormal frame. Used for defining
        default plot labels.
@@ -1515,9 +1564,10 @@ def remove_offset(
     # check and load kwargs
     valid.valid_kwargs(
         kwargs=kwargs,
-        allowed_kwargs={"reciprocal_space", "is_orthogonal"},
+        allowed_kwargs={"cmap", "reciprocal_space", "is_orthogonal"},
         name="postprocessing_utils.average_obj",
     )
+    cmap = kwargs.get("cmap", "turbo")
     valid.valid_container(
         offset_origin,
         container_types=(tuple, list, np.ndarray),
@@ -1536,6 +1586,7 @@ def remove_offset(
             title=title + " before offset removal",
             reciprocal_space=reciprocal_space,
             is_orthogonal=is_orthogonal,
+            cmap=cmap,
         )
 
     if phase_offset is None:
@@ -1549,11 +1600,9 @@ def remove_offset(
                 int(np.rint(ycom)),
                 int(np.rint(xcom)),
             )
-            print("\nCOM at pixels (z, y, x): ", zcom, ycom, xcom)
+            print(f"\nCOM at pixels (z, y, x): {zcom, ycom, xcom}")
             print(
-                "Offset at COM(support) of:",
-                str("{:.2f}".format(array[zcom, ycom, xcom])),
-                "rad",
+                f"Offset at COM(support) of: {array[zcom, ycom, xcom]:.2f} rad",
             )
             array = array - array[zcom, ycom, xcom] + phase_offset
         elif offset_method == "mean":
@@ -1568,13 +1617,8 @@ def remove_offset(
             offset_origin[2],
         )
         print(
-            "Offset of ",
-            str(
-                "{:.2f}".format(
-                    array[offset_origin[0], offset_origin[1], offset_origin[2]]
-                )
-            ),
-            "rad",
+            "Offset of "
+            f"{array[offset_origin[0], offset_origin[1], offset_origin[2]]:.2f} rad",
         )
         array = (
             array
@@ -1590,6 +1634,7 @@ def remove_offset(
             title=title + " after offset removal",
             reciprocal_space=reciprocal_space,
             is_orthogonal=is_orthogonal,
+            cmap=cmap,
         )
     return array
 
@@ -1606,6 +1651,7 @@ def remove_ramp(
     method="gradient",
     ups_factor=2,
     debugging=False,
+    **kwargs,
 ):
     """
     Remove the linear trend in the ramp using its gradient and a threshold n 3D dataset.
@@ -1628,15 +1674,21 @@ def remove_ramp(
      this value)
     :param debugging: set to True to see plots
     :type debugging: bool
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+     - 'logger': an optional logger
+
     :return: normalized amplitude, detrended phase, ramp along z, ramp along y,
      ramp along x
     """
+    logger = kwargs.get("logger", module_logger)
     valid.valid_ndarray(arrays=(amp, phase), ndim=3)
-
+    cmap = kwargs.get("cmap", "turbo")
     if method == "upsampling":
         nbz, nby, nbx = [mysize * ups_factor for mysize in initial_shape]
         nb_z, nb_y, nb_x = amp.shape
-        myobj = util.crop_pad(amp * np.exp(1j * phase), (nbz, nby, nbx))
+        myobj = util.crop_pad(amp * np.exp(1j * phase), (nbz, nby, nbx), cmap=cmap)
         if debugging:
             plt.figure()
             plt.imshow(np.log10(abs(myobj).sum(axis=0)))
@@ -1648,12 +1700,12 @@ def remove_ramp(
         if debugging:
             plt.figure()
             # plt.imshow(np.log10(abs(my_fft[nbz//2, :, :])))
-            plt.imshow(np.log10(abs(my_fft).sum(axis=0)))
+            plt.imshow(np.log10(abs(my_fft).sum(axis=0)), cmap=cmap)
             plt.title("np.log10(abs(my_fft).sum(axis=0))")
             plt.pause(0.1)
         zcom, ycom, xcom = center_of_mass(abs(my_fft) ** 4)
-        print("FFT shape for subpixel shift:", nbz, nby, nbx)
-        print("COM before subpixel shift", zcom, ",", ycom, ",", xcom)
+        logger.info(f"FFT shape for subpixel shift: ({nbz}, {nby}, {nbx})")
+        logger.info(f"COM before subpixel shift: ({zcom}, {ycom}, {xcom})")
         shiftz = zcom - (nbz / 2)
         shifty = ycom - (nby / 2)
         shiftx = xcom - (nbx / 2)
@@ -1664,7 +1716,7 @@ def remove_ramp(
         gc.collect()
         if debugging:
             plt.figure()
-            plt.imshow(abs(buf2ft).sum(axis=0))
+            plt.imshow(abs(buf2ft).sum(axis=0), cmap=cmap)
             plt.title("abs(buf2ft).sum(axis=0)")
             plt.pause(0.1)
 
@@ -1682,7 +1734,7 @@ def remove_ramp(
         gc.collect()
         if debugging:
             plt.figure()
-            plt.imshow(abs(greg).sum(axis=0))
+            plt.imshow(abs(greg).sum(axis=0), cmap=cmap)
             plt.title("abs(greg).sum(axis=0)")
             plt.pause(0.1)
 
@@ -1693,17 +1745,17 @@ def remove_ramp(
 
         if debugging:
             plt.figure()
-            plt.imshow(np.log10(abs(my_fft).sum(axis=0)))
+            plt.imshow(np.log10(abs(my_fft).sum(axis=0)), cmap=cmap)
             plt.title("centered np.log10(abs(my_fft).sum(axis=0))")
             plt.pause(0.1)
 
-        print("COM after subpixel shift", center_of_mass(abs(my_fft) ** 4))
+        logger(f"COM after subpixel shift: {center_of_mass(abs(my_fft) ** 4)}")
         myobj = fftshift(ifftn(ifftshift(my_fft)))
         del my_fft
         gc.collect()
         if debugging:
             plt.figure()
-            plt.imshow(abs(myobj).sum(axis=0))
+            plt.imshow(abs(myobj).sum(axis=0), cmap=cmap)
             plt.title("centered abs(myobj).sum(axis=0)")
             plt.pause(0.1)
 
@@ -1711,11 +1763,8 @@ def remove_ramp(
             myobj, (nb_z, nb_y, nb_x)
         )  # return to the initial shape of myamp
         print(
-            "Upsampling: shift_z, shift_y, shift_x: (",
-            str("{:.3f}".format(shiftz)),
-            str("{:.3f}".format(shifty)),
-            str("{:.3f}".format(shiftx)),
-            ") pixels",
+            "Upsampling: shift_z, shift_y, shift_x: "
+            f"({shiftz:.3f}, {shifty:.3f}, {shiftx:.3f}) pixels"
         )
         return abs(myobj) / abs(myobj).max(), np.angle(myobj), shiftz, shifty, shiftx
 
@@ -1746,6 +1795,7 @@ def remove_ramp(
             vmin=-threshold_gradient,
             vmax=threshold_gradient,
             title="Phase gradient along Z",
+            cmap=cmap,
         )
         gu.multislices_plot(
             mysupportz,
@@ -1755,6 +1805,7 @@ def remove_ramp(
             vmin=0,
             vmax=1,
             title="Thresholded support along Z",
+            cmap=cmap,
         )
     del mysupportz, mygradz
     gc.collect()
@@ -1779,6 +1830,7 @@ def remove_ramp(
             vmin=-threshold_gradient,
             vmax=threshold_gradient,
             title="Phase gradient along Y",
+            cmap=cmap,
         )
         gu.multislices_plot(
             mysupporty,
@@ -1788,6 +1840,7 @@ def remove_ramp(
             vmin=0,
             vmax=1,
             title="Thresholded support along Y",
+            cmap=cmap,
         )
     del mysupporty, mygrady
     gc.collect()
@@ -1812,6 +1865,7 @@ def remove_ramp(
             vmin=-threshold_gradient,
             vmax=threshold_gradient,
             title="Phase gradient along X",
+            cmap=cmap,
         )
         gu.multislices_plot(
             mysupportx,
@@ -1821,6 +1875,7 @@ def remove_ramp(
             vmin=0,
             vmax=1,
             title="Thresholded support along X",
+            cmap=cmap,
         )
     del mysupportx, mygradx, mysupport
     gc.collect()
@@ -1832,9 +1887,9 @@ def remove_ramp(
         indexing="ij",
     )
 
-    print(
-        "Gradient: phase_ramp_z, phase_ramp_y, phase_ramp_x: ",
-        f"({myrampz:.3f} rad, {myrampy:.3f} rad, {myrampx:.3f} rad)",
+    logger.info(
+        "Gradient: phase_ramp_z, phase_ramp_y, phase_ramp_x: "
+        f"({myrampz:.3f} rad, {myrampy:.3f} rad, {myrampx:.3f} rad)"
     )
     phase = phase - myz * myrampz - myy * myrampy - myx * myrampx
     return amp, phase, myrampz, myrampy, myrampx
@@ -1949,12 +2004,7 @@ def remove_ramp_2d(
         myobj = util.crop_pad_2d(
             myobj, (nb_y, nb_x)
         )  # return to the initial shape of myamp
-        print(
-            "Upsampling: shift_y, shift_x: (",
-            str("{:.3f}".format(shifty)),
-            str("{:.3f}".format(shiftx)),
-            ") pixels",
-        )
+        print(f"Upsampling: (shift_y, shift_x) = ({shifty:.3f},{shiftx:.3f}) pixels")
         return abs(myobj) / abs(myobj).max(), np.angle(myobj)
 
     # method='gradient'
@@ -2017,12 +2067,7 @@ def remove_ramp_2d(
 
     myy, myx = np.meshgrid(np.arange(0, nby, 1), np.arange(0, nbx, 1), indexing="ij")
 
-    print(
-        "Gradient: Phase_ramp_z, Phase_ramp_y, Phase_ramp_x: (",
-        str("{:.3f}".format(myrampy)),
-        str("{:.3f}".format(myrampx)),
-        ") rad",
-    )
+    print(f"Gradient: (Phase_ramp_y, Phase_ramp_x) = ({myrampy:.3f},{myrampx:.3f}) rad")
     phase = phase - myy * myrampy - myx * myrampx
     return amp, phase, myrampy, myrampx
 
@@ -2146,7 +2191,9 @@ def tukey_window(shape, alpha=np.array([0.5, 0.5, 0.5])):
     return tukey3
 
 
-def unwrap(obj, support_threshold, seed=0, debugging=True, **kwargs):
+def unwrap(
+    obj, support_threshold, seed=0, skip_unwrap: bool = False, debugging=True, **kwargs
+):
     """
     Unwrap the phase of a complex object.
 
@@ -2157,8 +2204,12 @@ def unwrap(obj, support_threshold, seed=0, debugging=True, **kwargs):
     :param support_threshold: relative threshold used to define a support from abs(obj)
     :param seed: int, random seed. Use always the same value if you want a
      deterministic behavior.
+    :param skip_unwrap: True to apply only the support_threshold to the phase but skip
+     the unwrapping part.
     :param debugging: set to True to see plots
     :param kwargs:
+
+     - 'cmap': str, name of the colormap
      - 'reciprocal_space': True if the object is in reciprocal space
      - 'is_orthogonal': True if the data is in an orthonormal frame.
        Used for defining default plot labels.
@@ -2174,7 +2225,7 @@ def unwrap(obj, support_threshold, seed=0, debugging=True, **kwargs):
     # check and load kwargs
     valid.valid_kwargs(
         kwargs=kwargs,
-        allowed_kwargs={"reciprocal_space", "is_orthogonal"},
+        allowed_kwargs={"cmap", "reciprocal_space", "is_orthogonal"},
         name="postprocessing_utils.average_obj",
     )
     reciprocal_space = kwargs.get("reciprocal_space", False)
@@ -2182,29 +2233,35 @@ def unwrap(obj, support_threshold, seed=0, debugging=True, **kwargs):
 
     ndim = obj.ndim
     unwrap_support = np.ones(obj.shape, dtype=int)
-    unwrap_support[
-        abs(obj) > support_threshold * abs(obj).max()
-    ] = 0  # 0 is a valid entry for ma.masked_array
-    phase_wrapped = ma.masked_array(np.angle(obj), mask=unwrap_support)
+    unwrap_support[abs(obj) > support_threshold * abs(obj).max()] = 0
+    # 0 is a valid entry for ma.masked_array
+    phase_wrapped: np.ndarray = ma.masked_array(np.angle(obj), mask=unwrap_support)
 
+    plot_title = "applying support threshold\n" if skip_unwrap else "unwrapping"
     if debugging and ndim == 3:
         gu.multislices_plot(
             phase_wrapped.data,
             plot_colorbar=True,
-            title="Object before unwrapping",
+            title=f"Before {plot_title}",
             reciprocal_space=reciprocal_space,
             is_orthogonal=is_orthogonal,
+            cmap=kwargs.get("cmap", "turbo"),
         )
 
-    phase_unwrapped = unwrap_phase(phase_wrapped, wrap_around=False, seed=seed).data
+    if skip_unwrap:
+        phase_unwrapped = phase_wrapped
+    else:
+        phase_unwrapped = unwrap_phase(phase_wrapped, wrap_around=False, seed=seed).data
+
     phase_unwrapped[np.nonzero(unwrap_support)] = 0
     if debugging and ndim == 3:
         gu.multislices_plot(
             phase_unwrapped,
             plot_colorbar=True,
-            title="Object after unwrapping",
+            title=f"After {plot_title}",
             reciprocal_space=reciprocal_space,
             is_orthogonal=is_orthogonal,
+            cmap=kwargs.get("cmap", "turbo"),
         )
 
     extent_phase = np.ceil(phase_unwrapped.max() - phase_unwrapped.min())
