@@ -7,24 +7,33 @@
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 """Functions related to data loading, encoding, fitting, data manipulation."""
 
-from collections import OrderedDict
 import ctypes
-from functools import reduce
 import gc
 import json
-import h5py
-from matplotlib import pyplot as plt
-from numbers import Real, Integral
-import numpy as np
+import logging
 import os
+import shutil
+from collections import OrderedDict
+from functools import reduce
+from inspect import signature
+from logging import Logger
+from numbers import Integral, Real
+from pathlib import Path
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
+
+import h5py
+import numpy as np
+from matplotlib import pyplot as plt
 from PIL import Image
-from scipy.interpolate import interp1d, RegularGridInterpolator
+from scipy.interpolate import RegularGridInterpolator, interp1d
 from scipy.optimize import curve_fit
 from scipy.special import erf
 from scipy.stats import multivariate_normal
-from typing import Union, Sequence
-from ..graph import graph_utils as gu
-from ..utils import validation as valid
+
+from bcdi.graph import graph_utils as gu
+from bcdi.utils import validation as valid
+
+module_logger = logging.getLogger(__name__)
 
 
 class CustomEncoder(json.JSONEncoder):
@@ -33,7 +42,7 @@ class CustomEncoder(json.JSONEncoder):
     def default(self, obj):
         """Override the JSONEncoder.default method to support more types."""
         if isinstance(obj, np.ndarray):
-            return CustomEncoder.ndarray_to_list(obj)
+            return ndarray_to_list(obj)
             # Let the base class default method raise the TypeError
         if isinstance(obj, set):
             return list(obj)
@@ -41,18 +50,6 @@ class CustomEncoder(json.JSONEncoder):
             return int(obj)
         return json.JSONEncoder.default(self, obj)
 
-    @staticmethod
-    def ndarray_to_list(obj):
-        """Convert a numpy ndarray of any dimension to a nested list."""
-        if not isinstance(obj, np.ndarray):
-            raise TypeError("a numpy ndarray is expected")
-        if obj.ndim == 1:
-            return list(obj)
-        output = []
-        for idx in range(obj.shape[0]):
-            output.append(CustomEncoder.ndarray_to_list(obj[idx]))
-        return output
-
 
 def apply_logical_array(arrays, frames_logical):
     """
@@ -100,7 +97,7 @@ def apply_logical_array(arrays, frames_logical):
     return output
 
 
-def bin_data(array, binning, debugging=False):
+def bin_data(array, binning, debugging=False, **kwargs):
     """
     Rebin a 1D, 2D or 3D array.
 
@@ -112,8 +109,15 @@ def bin_data(array, binning, debugging=False):
      binning (x binning (x binning)). This can also be a tuple/list of rebin values
      along each axis, e.g. binning=(4,1,2) for a 3D array
     :param debugging: boolean, True to see plots
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+     - 'logger': an optional logger
+
     :return: the binned array
     """
+    cmap = kwargs.get("cmap", "turbo")
+    logger = kwargs.get("logger", module_logger)
     valid.valid_ndarray(arrays=array, ndim=(1, 2, 3))
     ndim = array.ndim
     if isinstance(binning, int):
@@ -153,8 +157,8 @@ def bin_data(array, binning, debugging=False):
         raise ValueError("Array should be 1D, 2D, or 3D")
 
     if debugging:
-        print("array shape after cropping but before binning:", array.shape)
-        print("array shape after binning:", newarray.shape)
+        logger.info(f"array shape after cropping but before binning: {array.shape}")
+        logger.info(f"array shape after binning: {newarray.shape}")
         gu.combined_plots(
             tuple_array=(array, newarray),
             tuple_sum_frames=False,
@@ -167,6 +171,7 @@ def bin_data(array, binning, debugging=False):
             tuple_title=("array", "binned array"),
             tuple_scale="log",
             reciprocal_space=True,
+            cmap=cmap,
         )
     return newarray
 
@@ -218,6 +223,32 @@ def bin_parameters(binning, nb_frames, params, debugging=True):
     return params
 
 
+def cast(
+    val: Union[float, List, np.ndarray], target_type: type = float
+) -> Union[float, List, np.ndarray]:
+    """
+    Cast val to a number or an array of numbers of the target type.
+
+    :param val: the value to be converted
+    :param target_type: the type to convert to
+    """
+    if not isinstance(target_type, type):
+        raise TypeError("target_type should be a type")
+    if target_type not in [int, float]:
+        raise ValueError(f"target_type should be 'int' or 'float', got {target_type}")
+    try:
+        if isinstance(val, np.ndarray):
+            val = val.astype(target_type)
+        elif isinstance(val, (list, tuple)):
+            val = [cast(value, target_type=target_type) for value in val]
+        else:
+            val = target_type(val)
+        return val
+    except (TypeError, ValueError):
+        print(f"Cannot cast to {target_type}")
+        raise
+
+
 def catch_error(exception):
     """
     Process exception in asynchronous multiprocessing.
@@ -227,8 +258,50 @@ def catch_error(exception):
     print(exception)
 
 
+def convert_str_target(
+    value: Any, target: str, conversion_table: Optional[Dict[str, Any]] = None
+) -> Any:
+    """
+    Convert strings from value to the desired target.
+
+    :param value: an object containing strings to be converted
+    :param target: the target string, which has to be present in the conversion table
+    :param conversion_table: a dictionary for the conversion
+    :return: the converted object
+    """
+    conversion_table = (
+        conversion_table
+        if conversion_table is not None
+        else {"none": None, "true": True, "false": False}
+    )
+    target = target.lower()
+    if target not in conversion_table:
+        raise ValueError(
+            f"invalid target {target}, valid targets: {list(conversion_table.keys())}"
+        )
+    if isinstance(value, str) and value.lower() == target:
+        return conversion_table[target]
+    if isinstance(value, (list, tuple)):
+        new_value = list(value)
+        for idx, val in enumerate(new_value):
+            new_value[idx] = convert_str_target(val, target=target)
+        return new_value
+    if isinstance(value, dict):
+        for key, item in value.items():
+            value[key] = convert_str_target(item, target=target)
+        return value
+
+    return value
+
+
 def crop_pad(
-    array, output_shape, pad_value=0, pad_start=None, crop_center=None, debugging=False
+    array,
+    output_shape,
+    pad_value=0,
+    pad_start=None,
+    crop_center=None,
+    debugging=False,
+    **kwargs,
 ):
     """
     Crop or pad the 3D object depending on output_shape.
@@ -243,8 +316,14 @@ def crop_pad(
      center of the original array
     :param debugging: set to True to see plots
     :type debugging: bool
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+     - 'logger': an optional logger
+
     :return: myobj cropped or padded with zeros
     """
+    logger = kwargs.get("logger", module_logger)
     valid.valid_ndarray(arrays=array, ndim=3)
     valid.valid_container(
         output_shape,
@@ -267,9 +346,13 @@ def crop_pad(
         raise ValueError("crop_center should be a list or tuple of three indices")
 
     if debugging:
-        print(f"array shape before crop/pad = {array.shape}")
+        logger.info(f"array shape before crop/pad = {array.shape}")
         gu.multislices_plot(
-            abs(array), sum_frames=True, scale="log", title="Before crop/pad"
+            abs(array),
+            sum_frames=True,
+            scale="log",
+            title="Before crop/pad",
+            cmap=kwargs.get("cmap", "turbo"),
         )
 
     # crop/pad along axis 0
@@ -312,9 +395,13 @@ def crop_pad(
         ]
 
     if debugging:
-        print(f"array shape after crop/pad = {newobj.shape}")
+        logger.info(f"array shape after crop/pad = {newobj.shape}")
         gu.multislices_plot(
-            abs(newobj), sum_frames=True, scale="log", title="After crop/pad"
+            abs(newobj),
+            sum_frames=True,
+            scale="log",
+            title="After crop/pad",
+            cmap=kwargs.get("cmap", "turbo"),
         )
     return newobj
 
@@ -465,23 +552,26 @@ def decode_json(dct):
     return dct
 
 
-def dos2unix(input_file, output_file):
+def dos2unix(input_file: str, savedir: str) -> None:
     """
     Convert DOS linefeeds (crlf) to UNIX (lf).
 
     :param input_file: the original filename (absolute path)
-    :param output_file: the output filename (absolute path) where to save
+    :param savedir: path of the directory where to save
     """
+    filename = os.path.splitext(os.path.basename(input_file))[0]
+    if not input_file.endswith(".txt"):
+        raise ValueError("Expecting a text file as input, e.g. 'alias_dict.txt'")
     with open(input_file, "rb") as infile:
         content = infile.read()
-    with open(output_file, "wb") as output:
+    with open(os.path.join(savedir, f"{filename}_unix.txt"), "wb") as output:
         for row in content.splitlines():
             output.write(row + str.encode("\n"))
 
 
-def find_file(filename: str, default_folder: str) -> str:
+def find_file(filename: Optional[str], default_folder: str, **kwargs) -> str:
     """
-    Localize a file.
+    Locate a file.
 
     The filename can be either the name of the file (including the extension) or the
     full path to the file.
@@ -489,15 +579,20 @@ def find_file(filename: str, default_folder: str) -> str:
     :param filename: the name or full path to the file
     :param default_folder: it will look for the file in that folder if filename is not
      the full path.
+    :param kwargs:
+
+     - 'logger': an optional logger
+
     :return: str, the path to the file
     """
+    logger = kwargs.get("logger", module_logger)
     if not isinstance(filename, str):
         raise TypeError("filename should be a string")
 
     if os.path.isfile(filename):
         # filename is already the full path to the file
         return filename
-    print(f"Could not find the file at: {filename}")
+    logger.info(f"Could not find the file at: {filename}")
 
     if not isinstance(default_folder, str):
         raise TypeError("default_folder should be a string")
@@ -509,7 +604,7 @@ def find_file(filename: str, default_folder: str) -> str:
     full_name = default_folder + filename
     if not os.path.isfile(full_name):
         raise ValueError(f"Could not localize the file at {filename} or {full_name}")
-    print(f"File localized at: {full_name}")
+    logger.info(f"File localized at: {full_name}")
     return full_name
 
 
@@ -657,6 +752,55 @@ def fit3d_poly4(x_axis, a, b, c, d, e, f, g, h, i, j, k, m, n):
     )
 
 
+def create_repr(obj: Any, cls: type) -> str:
+    """
+    Generate the string representation of the object.
+
+    It uses the parameters given to __init__, except self, args and kwargs.
+
+    :param obj: the object for which the string representation should be generated
+    :param cls: the cls from which __init__ parameters should be extracted (e.g., base
+     class in case of inheritance)
+    :return: the string representation
+    """
+    if not isinstance(cls, type):
+        raise TypeError(f"'cls' should be a class, for {type(cls)}")
+    output = obj.__class__.__name__ + "("
+    for _, param in enumerate(
+        signature(cls.__init__).parameters.keys()  # type: ignore
+    ):
+        quote_mark = True
+        if param not in ["self", "args", "kwargs"]:
+            value = getattr(obj, param)
+            if isinstance(value, np.ndarray):
+                value = ndarray_to_list(value)
+            if callable(value):
+                value = value.__module__ + "." + value.__name__
+                # it's a string but we don't want to put it in quote mark in order to
+                # be able to call it directly
+                quote_mark = False
+            output += format_repr(param, value, quote_mark=quote_mark)
+
+    output += ")"
+    return str(output)
+
+
+def format_repr(field: str, value: Optional[Any], quote_mark: bool = True) -> str:
+    """
+    Format a string for the __repr__ method depending on its value.
+
+    :param field: str, the value of the field in __repr__
+    :param value: string or None
+    :param quote_mark: True to put quote marks around strings
+    :return: a string
+    """
+    if not isinstance(field, str):
+        raise TypeError(f"'field should be a string, got {type(field)}'")
+    if isinstance(value, str) and quote_mark:
+        return f'{field}="{value}", '.replace("\\", "/")
+    return f"{field}={value}, ".replace("\\", "/")
+
+
 def function_lmfit(params, x_axis, distribution, iterator=0):
     """
     Calculate distribution defined by lmfit Parameters.
@@ -668,26 +812,26 @@ def function_lmfit(params, x_axis, distribution, iterator=0):
     :return: the gaussian function calculated at x_axis positions
     """
     if distribution == "gaussian":
-        amp = params["amp_%i" % (iterator + 1)].value
-        cen = params["cen_%i" % (iterator + 1)].value
-        sig = params["sig_%i" % (iterator + 1)].value
+        amp = params[f"amp_{iterator}"].value
+        cen = params[f"cen_{iterator}"].value
+        sig = params[f"sig_{iterator}"].value
         return gaussian(x_axis=x_axis, amp=amp, cen=cen, sig=sig)
     if distribution == "skewed_gaussian":
-        amp = params["amp_%i" % (iterator + 1)].value
-        loc = params["loc_%i" % (iterator + 1)].value
-        sig = params["sig_%i" % (iterator + 1)].value
-        alpha = params["alpha_%i" % (iterator + 1)].value
+        amp = params[f"amp_{iterator}"].value
+        loc = params[f"loc_{iterator}"].value
+        sig = params[f"sig_{iterator}"].value
+        alpha = params[f"alpha_{iterator}"].value
         return skewed_gaussian(x_axis=x_axis, amp=amp, loc=loc, sig=sig, alpha=alpha)
     if distribution == "lorentzian":
-        amp = params["amp_%i" % (iterator + 1)].value
-        cen = params["cen_%i" % (iterator + 1)].value
-        sig = params["sig_%i" % (iterator + 1)].value
+        amp = params[f"amp_{iterator}"].value
+        cen = params[f"cen_{iterator}"].value
+        sig = params[f"sig_{iterator}"].value
         return lorentzian(x_axis=x_axis, amp=amp, cen=cen, sig=sig)
     if distribution == "pseudovoigt":
-        amp = params["amp_%i" % (iterator + 1)].value
-        cen = params["cen_%i" % (iterator + 1)].value
-        sig = params["sig_%i" % (iterator + 1)].value
-        ratio = params["ratio_%i" % (iterator + 1)].value
+        amp = params[f"amp_{iterator}"].value
+        cen = params[f"cen_{iterator}"].value
+        sig = params[f"sig_{iterator}"].value
+        ratio = params[f"ratio_{iterator}"].value
         return pseudovoigt(x_axis, amp=amp, cen=cen, sig=sig, ratio=ratio)
     raise ValueError(distribution + " not implemented")
 
@@ -702,7 +846,7 @@ def gaussian(x_axis, amp, cen, sig):
     :param sig: HWHM of the Gaussian
     :return: the Gaussian line shape at x_axis
     """
-    return amp * np.exp(-((x_axis - cen) ** 2) / (2.0 * sig ** 2))
+    return amp * np.exp(-((x_axis - cen) ** 2) / (2.0 * sig**2))
 
 
 def gaussian_window(window_shape, sigma=0.3, mu=0.0, voxel_size=None, debugging=False):
@@ -756,7 +900,7 @@ def gaussian_window(window_shape, sigma=0.3, mu=0.0, voxel_size=None, debugging=
             np.linspace(-nbx, nbx, nbx) * voxel_size[1],
             indexing="ij",
         )
-        covariance = np.diag(sigma ** 2)
+        covariance = np.diag(sigma**2)
         window = multivariate_normal.pdf(
             np.column_stack([grid_y.flat, grid_x.flat]), mean=mu, cov=covariance
         )
@@ -772,7 +916,7 @@ def gaussian_window(window_shape, sigma=0.3, mu=0.0, voxel_size=None, debugging=
             np.linspace(-nbx, nbx, nbx) * voxel_size[2],
             indexing="ij",
         )
-        covariance = np.diag(sigma ** 2)
+        covariance = np.diag(sigma**2)
         window = multivariate_normal.pdf(
             np.column_stack([grid_z.flat, grid_y.flat, grid_x.flat]),
             mean=mu,
@@ -848,7 +992,7 @@ def higher_primes(number, maxprime=13, required_dividers=(4,)):
     return number
 
 
-def image_to_ndarray(filename, convert_grey=True, cmap=None, debug=False):
+def image_to_ndarray(filename, convert_grey=True):
     """
     Convert an image to a numpy array using pillow.
 
@@ -857,25 +1001,14 @@ def image_to_ndarray(filename, convert_grey=True, cmap=None, debug=False):
     :param filename: absolute path of the image to open
     :param convert_grey: if True and the number of layers is 3, it will be converted
      to a single layer of grey
-    :param cmap: colormap for the plots
-    :param debug: True to see plots
     :return:
     """
-    if cmap is None:
-        cmap = gu.Colormap(bad_color="1.0").cmap
-
     im = Image.open(filename)
 
     array = np.asarray(im)
     if array.ndim == 3 and convert_grey:
         print("converting image to gray")
         array = rgb2gray(array)
-
-    if debug:
-        print(f"Image shape after conversion to ndarray: {array.shape}")
-        gu.imshow_plot(
-            array, sum_axis=2, plot_colorbar=True, cmap=cmap, reciprocal_space=False
-        )
     return array
 
 
@@ -929,25 +1062,9 @@ def in_range(point, extent):
     return False
 
 
-def is_float(string):
-    """
-    Return True is the string represents a number.
-
-    :param string: the string to be checked
-    :return: True of False
-    """
-    if not isinstance(string, str):
-        raise TypeError("the input should be a string")
-    try:
-        float(string)
-        return True
-    except ValueError:
-        return False
-
-
 def linecut(array, point, direction, direction_basis="voxel", voxel_size=1):
     """
-    Calculate the linecut through a 2D or 3D array in some direction passing by a point.
+    Calculate iteratively a linecut through an array without interpolation.
 
     :param array: 2D or 3D numpy array from which the linecut will be extracted
     :param point: tuple of three integral indices, point by which the linecut pass.
@@ -1098,17 +1215,16 @@ def load_background(background_file):
     :return: a 2D background
     """
     if background_file:
-        background = np.load(background_file)
-        if background_file.endswith("npz"):
-            npz_key = background.files
-            background = background[npz_key[0]]
-            valid.valid_ndarray(background, ndim=2)
+        background, _ = load_file(background_file)
+        valid.valid_ndarray(background, ndim=2)
     else:
         background = None
     return background
 
 
-def load_file(file_path, fieldname=None):
+def load_file(
+    file_path: str, fieldname: Optional[str] = None
+) -> Tuple[np.ndarray, str]:
     """
     Load a file.
 
@@ -1122,34 +1238,33 @@ def load_file(file_path, fieldname=None):
     """
     _, extension = os.path.splitext(file_path)
     if extension == ".npz":  # could be anything
-        if fieldname is None:  # output of PyNX phasing
-            npzfile = np.load(file_path)
-            dataset = npzfile[list(npzfile.files)[0]]
-        else:  # could be anything
-            try:
-                dataset = np.load(file_path)[fieldname]
-                return dataset, extension
-            except KeyError:
-                npzfile = np.load(file_path)
+        with np.load(file_path) as npzfile:
+            if fieldname is None:  # output of PyNX phasing or flatfield/background
                 dataset = npzfile[list(npzfile.files)[0]]
+            else:  # could be anything
+                try:
+                    dataset = npzfile[fieldname]
+                except KeyError:
+                    dataset = npzfile[list(npzfile.files)[0]]
     elif extension == ".npy":  # could be anything
+        # no need to close the file for .npy extension, see np.load docstring
         dataset = np.load(file_path)
     elif extension == ".cxi":  # output of PyNX phasing
-        h5file = h5py.File(file_path, "r")
-        # group_key = list(h5file.keys())[1]
-        # subgroup_key = list(h5file[group_key])
-        # dataset = h5file['/'+group_key+'/'+subgroup_key[0]+'/data'].value
-        dataset = h5file["/entry_1/data_1/data"][()]
+        with h5py.File(file_path, "r") as h5file:
+            # group_key = list(h5file.keys())[1]
+            # subgroup_key = list(h5file[group_key])
+            # dataset = h5file['/'+group_key+'/'+subgroup_key[0]+'/data'].value
+            dataset = h5file["/entry_1/data_1/data"][()]
     elif extension == ".h5":  # modes.h5
-        h5file = h5py.File(file_path, "r")
-        group_key = list(h5file.keys())[0]
-        if group_key == "mask":  # mask object for Nanomax data
-            dataset = h5file["/" + group_key][:]
-        else:  # modes.h5 file output of PyNX phase retrieval
-            subgroup_key = list(h5file[group_key])
-            dataset = h5file["/" + group_key + "/" + subgroup_key[0] + "/data"][
-                0
-            ]  # select only first mode
+        with h5py.File(file_path, "r") as h5file:
+            group_key = list(h5file.keys())[0]
+            if group_key == "mask":  # mask object for Nanomax data
+                dataset = h5file["/" + group_key][:]
+            else:  # modes.h5 file output of PyNX phase retrieval
+                subgroup_key = list(h5file[group_key])
+                dataset = h5file["/" + group_key + "/" + subgroup_key[0] + "/data"][
+                    0
+                ]  # select only first mode
     else:
         raise ValueError(
             "File format not supported: "
@@ -1174,12 +1289,9 @@ def load_flatfield(flatfield_file):
     :param flatfield_file: the path of the flatfield file
     :return: a 2D flatfield
     """
-    if flatfield_file:
-        flatfield = np.load(flatfield_file)
-        if flatfield_file.endswith(".npz"):
-            npz_key = flatfield.files
-            flatfield = flatfield[npz_key[0]]
-            valid.valid_ndarray(flatfield, ndim=2)
+    if flatfield_file and os.path.isfile(flatfield_file):
+        flatfield, _ = load_file(flatfield_file)
+        valid.valid_ndarray(flatfield, ndim=2)
     else:
         flatfield = None
     return flatfield
@@ -1220,7 +1332,7 @@ def lorentzian(x_axis, amp, cen, sig):
     :param sig: HWHM of the Lorentzian
     :return: the Lorentzian line shape at x_axis
     """
-    return amp / (sig * np.pi) / (1 + (x_axis - cen) ** 2 / (sig ** 2))
+    return amp / (sig * np.pi) / (1 + (x_axis - cen) ** 2 / (sig**2))
 
 
 def make_support(
@@ -1269,6 +1381,7 @@ def mean_filter(
     min_count=3,
     interpolate="mask_isolated",
     debugging=False,
+    **kwargs,
 ):
     """
     Mask or apply a mean filter to data.
@@ -1289,8 +1402,13 @@ def mean_filter(
      isolated pixels, if 'interp_isolated' will interpolate isolated pixels
     :param debugging: set to True to see plots
     :type debugging: bool
+    :param kwargs:
+
+     - 'cmap': str, name of the colormap
+
     :return: updated data and mask, number of pixels treated
     """
+    cmap = kwargs.get("cmap", "turbo")
     valid.valid_ndarray(arrays=data, ndim=(2, 3))
     # check some mparameters
     if mask is None:
@@ -1336,6 +1454,7 @@ def mean_filter(
             tuple_scale=("log", "linear"),
             tuple_title=("Data before filtering", "Mask before filtering"),
             reciprocal_space=True,
+            cmap=cmap,
         )
 
     if data.ndim == 2:
@@ -1390,11 +1509,43 @@ def mean_filter(
             tuple_scale=("log", "linear"),
             tuple_title=("Data after filtering", "Mask after filtering"),
             reciprocal_space=True,
+            cmap=cmap,
         )
 
     return data, nb_pixels, mask
 
 
+def move_log(result: Tuple[Path, Path, Optional[Logger]]):
+    """
+    Move log files to the desired location, after processing a file.
+
+    It can be used as a standard function or as a callback for multiprocessing.
+
+    :param result: the output of process_scan, containing the 2d data, 2d mask,
+     counter for each frame; the file index; and an optional logger
+    """
+    logger = result[2] if result[2] is not None else module_logger
+    filename = result[0].name
+    shutil.move(result[0], result[1] / filename)
+    logger.info(f"{filename.replace('.log', '')} processed")
+
+
+def ndarray_to_list(array: np.ndarray) -> List:
+    """
+    Convert a numpy ndarray of any dimension to a nested list.
+
+    :param array: the array to be converted
+    """
+    if not isinstance(array, np.ndarray):
+        raise TypeError("a numpy ndarray is expected")
+    if array.ndim == 1:
+        return list(array)
+    output = []
+    for idx in range(array.shape[0]):
+        output.append(ndarray_to_list(array[idx]))
+    return output
+
+
 def objective_lmfit(params, x_axis, data, distribution):
     """
     Calculate the total residual for fits to several data sets.
@@ -1407,19 +1558,21 @@ def objective_lmfit(params, x_axis, data, distribution):
     :param distribution: distribution to use for fitting
     :return: the residuals of the fit of data using the parameters
     """
-    if len(data.shape) == 1:  # single dataset
+    ndim = data.ndim
+    if ndim == 1 and not isinstance(data[0], np.ndarray):  # single dataset
         data = data[np.newaxis, :]
         x_axis = x_axis[np.newaxis, :]
-    if data.ndim != 2:
-        raise ValueError("Data should be a 2D stack of 1D datasets (1 per row)")
-    ndata, _ = data.shape
+    if ndim not in [1, 2]:
+        raise ValueError(f"data should be 1D or 2D, got {ndim}D")
+
+    ndata = data.shape[0]
     resid = 0.0 * data[:]
     # make residual per data set
     for idx in range(ndata):
-        resid[idx, :] = data[idx, :] - function_lmfit(
+        resid[idx] = data[idx] - function_lmfit(
             params=params,
             iterator=idx,
-            x_axis=x_axis[idx, :],
+            x_axis=x_axis[idx],
             distribution=distribution,
         )
     # now flatten this to a 1D array, as minimize() needs
@@ -1438,7 +1591,7 @@ def line(x_array, a, b):
     return a * x_array + b
 
 
-def pad_from_roi(arrays, roi, binning, pad_value=0):
+def pad_from_roi(arrays, roi, binning, pad_value=0, **kwargs):
     """
     Pad a 3D stack of frames provided a region of interest.
 
@@ -1449,9 +1602,13 @@ def pad_from_roi(arrays, roi, binning, pad_value=0):
      arrays, the shape is (nz, ny, nx), and roi corresponds to [y0, y1, x0, x1]
     :param binning: tuple of two integers (binning along Y, binning along X)
     :param pad_value: number or tuple of nb_arrays numbers, will pad using this value
+    :param kwargs:
+     - 'logger': an optional logger
+
     :return: an array (if a single array was provided) or a tuple of arrays interpolated
      on an orthogonal grid (same length as the number of input arrays)
     """
+    logger = kwargs.get("logger", module_logger)
     ####################
     # check parameters #
     ####################
@@ -1503,7 +1660,7 @@ def pad_from_roi(arrays, roi, binning, pad_value=0):
         else:  # padding on the right
             startx = 0
         start = [int(val) for val in [0, starty, startx]]
-        print("Paddind the data to the shape defined by the ROI")
+        logger.info("Paddind the data to the shape defined by the ROI")
 
         ##############
         # pad arrays #
@@ -1907,6 +2064,8 @@ def rotate_crystal(
     :param debugging: tuple of booleans of the same length as the number of
      input arrays, True to see plots before and after rotation
     :param kwargs:
+
+     - 'cmap': str, name of the colormap
      - 'title': tuple of strings, titles for the debugging plots, same length as the
        number of arrays
      - 'scale': tuple of strings (either 'linear' or 'log'), scale for the debugging
@@ -1993,9 +2152,10 @@ def rotate_crystal(
     # check and load kwargs
     valid.valid_kwargs(
         kwargs=kwargs,
-        allowed_kwargs={"title", "scale", "width_z", "width_y", "width_x"},
+        allowed_kwargs={"cmap", "title", "scale", "width_z", "width_y", "width_x"},
         name="Setup.orthogonalize",
     )
+    cmap = kwargs.get("cmap", "turbo")
     title = kwargs.get("title", ("Object",) * nb_arrays)
     valid.valid_container(
         title,
@@ -2107,6 +2267,7 @@ def rotate_crystal(
                 is_orthogonal=is_orthogonal,
                 scale=scale[idx],
                 reciprocal_space=reciprocal_space,
+                cmap=cmap,
             )
             gu.multislices_plot(
                 rotated_array,
@@ -2117,6 +2278,7 @@ def rotate_crystal(
                 is_orthogonal=is_orthogonal,
                 scale=scale[idx],
                 reciprocal_space=reciprocal_space,
+                cmap=cmap,
             )
 
     if nb_arrays == 1:
@@ -2238,7 +2400,7 @@ def skewed_gaussian(x_axis, amp, loc, sig, alpha):
     """
     return (
         amp
-        * np.exp(-((x_axis - loc) ** 2) / (2.0 * sig ** 2))
+        * np.exp(-((x_axis - loc) ** 2) / (2.0 * sig**2))
         * (1 + erf(alpha / np.sqrt(2) * (x_axis - loc) / sig))
     )
 
@@ -2361,6 +2523,55 @@ def try_smaller_primes(number, maxprime=13, required_dividers=(4,)):
     return True
 
 
+def unpack_array(
+    array: Union[float, List[float], np.ndarray]
+) -> Union[float, np.ndarray]:
+    """Unpack an array or Sequence of length 1 into a single element."""
+    if isinstance(array, (list, tuple, np.ndarray)) and len(array) == 1:
+        return array[0]
+    return np.asarray(array)
+
+
+def upsample(array: Union[np.ndarray, List], factor: int = 2) -> np.ndarray:
+    """
+    Upsample an array.
+
+    :param array: the numpy array to be upsampled
+    :param factor: int, factor for the upsampling
+    :return: the upsampled numpy array
+    """
+    # check parameters
+    array = np.asarray(array)
+    if array.dtype in ["int8", "int16", "int32", "int64"]:
+        array = array.astype(float)
+
+    valid.valid_item(factor, allowed_types=int, min_excluded=0, name="factor")
+
+    # current points positions in each dimension
+    old_positions = [np.arange(val) for val in array.shape]
+
+    # calculate the new positions
+    new_shape = [val * factor for val in array.shape]
+    upsampled_positions = [
+        np.linspace(0, val - 1, num=val * factor) for val in array.shape
+    ]
+    grid = np.meshgrid(*upsampled_positions, indexing="ij")
+    new_grid = np.asarray(
+        np.concatenate(
+            [new_grid.reshape((1, new_grid.size)) for _, new_grid in enumerate(grid)]
+        ).transpose()
+    )
+    # interpolate array #
+    rgi = RegularGridInterpolator(
+        old_positions,
+        array,
+        method="linear",
+        bounds_error=False,
+        fill_value=0,
+    )
+    return np.asarray(rgi(new_grid).reshape(new_shape).astype(array.dtype))
+
+
 def wrap(obj, start_angle, range_angle):
     """
     Wrap obj between start_angle and (start_angle + range_angle).
