@@ -7,10 +7,27 @@
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 """Functions related to the validation of input parameters."""
 
-from numbers import Real, Number
+from numbers import Number, Real
+
 import numpy as np
 
 
+def is_float(string):
+    """
+    Return True is the string represents a number.
+
+    :param string: the string to be checked
+    :return: True of False
+    """
+    if not isinstance(string, str):
+        raise TypeError("the input should be a string")
+    try:
+        float(string)
+        return True
+    except ValueError:
+        return False
+
+
 def valid_container(
     obj,
     container_types,
@@ -270,7 +287,7 @@ def valid_item(
     # check the type of obj
     if value is not None and not isinstance(value, allowed_types):
         raise TypeError(
-            f"{name}: wrong type for value, "
+            f"{name}: wrong type for value {value}, "
             f"allowed is {allowed_types}, got {type(value)}"
         )
 
