@@ -11,17 +11,18 @@ try:
     import hdf5plugin  # for P10, should be imported before h5py or PyTables
 except ModuleNotFoundError:
     pass
-import numpy as np
+import os
+
 import matplotlib
 import matplotlib.pyplot as plt
-from matplotlib.widgets import RectangleSelector
 import matplotlib.ticker as ticker
-import os
+import numpy as np
+from matplotlib.widgets import RectangleSelector
+
 import bcdi.graph.graph_utils as gu
 import bcdi.utils.utilities as util
-import bcdi.preprocessing.bcdi_utils as bu
-from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
+from bcdi.graph.colormap import ColormapFactory
 
 matplotlib.use("Qt5Agg")
 
@@ -244,40 +245,39 @@ def press_key(event):
 # define colormap #
 ###################
 bad_color = "1.0"  # white background
-colormap = gu.Colormap(bad_color=bad_color)
-my_cmap = colormap.cmap
+my_cmap = ColormapFactory(bad_color=bad_color).cmap
 plt.ion()
 
-#################################################
-# initialize detector, setup, paths and logfile #
-#################################################
-detector = create_detector(
-    name=detector,
+########################
+# initialize the setup #
+########################
+setup = Setup(
+    beamline_name=beamline,
+    is_series=is_series,
+    detector_name=detector,
     datadir="",
     template_imagefile=template_imagefile,
     sum_roi=sum_roi,
     binning=[1, binning[0], binning[1]],
 )
 
-setup = Setup(beamline=beamline, detector=detector, is_series=is_series)
-
 if setup.beamline == "P10":
     specfile_name = sample_name + "_{:05d}".format(scan)
     homedir = root_folder + specfile_name + "/"
-    detector.datadir = homedir + "e4m/"
+    setup.detector.datadir = homedir + "e4m/"
     template_imagefile = specfile_name + template_imagefile
-    detector.template_imagefile = template_imagefile
+    setup.detector.template_imagefile = template_imagefile
 elif setup.beamline in {"SIXS_2018", "SIXS_2019"}:
     homedir = root_folder
-    detector.datadir = homedir + "align/"
+    setup.detector.datadir = homedir + "align/"
 else:
     homedir = root_folder + sample_name + str(scan) + "/"
-    detector.datadir = homedir + "data/"
+    setup.detector.datadir = homedir + "data/"
 
 if savedir == "":
-    savedir = os.path.abspath(os.path.join(detector.datadir, os.pardir)) + "/"
+    savedir = os.path.abspath(os.path.join(setup.detector.datadir, os.pardir)) + "/"
 
-detector.savedir = savedir
+setup.detector.savedir = savedir
 print("savedir: ", savedir)
 
 logfile = setup.create_logfile(
@@ -288,13 +288,13 @@ logfile = setup.create_logfile(
 # check some parameters #
 #########################
 if len(sum_roi) == 0:
-    sum_roi = [0, detector.nb_pixel_y, 0, detector.nb_pixel_x]
+    sum_roi = [0, setup.detector.nb_pixel_y, 0, setup.detector.nb_pixel_x]
 
 if not (
     sum_roi[0] >= 0
-    and sum_roi[1] <= detector.nb_pixel_y // binning[0]
+    and sum_roi[1] <= setup.detector.nb_pixel_y // binning[0]
     and sum_roi[2] >= 0
-    and sum_roi[3] <= detector.nb_pixel_x // binning[1]
+    and sum_roi[3] <= setup.detector.nb_pixel_x // binning[1]
 ):
     raise ValueError("sum_roi setting does not match the binned detector size")
 
@@ -304,9 +304,8 @@ if scale not in {"linear", "log"}:
 #############
 # load data #
 #############
-data, mask, monitor, frames_logical = setup.diffractometer.load_check_dataset(
+data, mask, monitor, frames_logical = setup.loader.load_check_dataset(
     scan_number=scan,
-    detector=detector,
     setup=setup,
     bin_during_loading=True,
     debugging=False,
@@ -318,8 +317,8 @@ data[np.nonzero(mask)] = 0
 ########################
 # load motor positions #
 ########################
-motor_positions = setup.diffractometer.read_device(
-    logfile=logfile, scan_number=scan, setup=setup, motor_name=motor_name
+motor_positions = setup.loader.read_device(
+    setup=setup, device_name=motor_name, scan_number=scan
 )
 
 min_fast, max_fast = motor_positions[0], motor_positions[-1]
