@@ -12,21 +12,24 @@ try:
 except ModuleNotFoundError:
     pass
 import gc
-from matplotlib import pyplot as plt
-import numpy as np
-from numpy.fft import fftn, fftshift
-from scipy.interpolate import interp1d
-from scipy.ndimage.measurements import center_of_mass
 import sys
 import tkinter as tk
 from tkinter import filedialog
+
+import numpy as np
 import xrayutilities as xu
+from matplotlib import pyplot as plt
+from numpy.fft import fftn, fftshift
+from scipy.interpolate import interp1d
+from scipy.ndimage.measurements import center_of_mass
+
 import bcdi.graph.graph_utils as gu
-from bcdi.experiment.detector import create_detector
-from bcdi.experiment.setup import Setup
+import bcdi.graph.linecut as lc
 import bcdi.utils.image_registration as reg
 import bcdi.utils.utilities as util
 import bcdi.utils.validation as valid
+from bcdi.experiment.setup import Setup
+from bcdi.graph.colormap import ColormapFactory
 
 helptext = """
 Calculate the resolution of a BCDI reconstruction using the phase retrieval transfer
@@ -177,12 +180,10 @@ def on_click(event):
         res_text.remove()
         res_text = fig_prtf.text(0.55, 0.25, "", size=10)
         print(f"endpoint = {endpoint}")
-        cut = gu.linecut(
+        cut = lc.linecut(
             linecut_prtf,
-            start_indices=(z0, y0, x0),
-            stop_indices=endpoint,
+            indices=list(zip(starting_point, endpoint)),
             interp_order=1,
-            debugging=False,
         )
         plt0.remove()
         plt1.remove()
@@ -231,24 +232,11 @@ def press_key(event):
         pass
 
 
-#######################
-# Initialize detector #
-#######################
-# phasing_binning will be taken into account after the optional data cropping
-# (crop_roi parameter)
-detector = create_detector(
-    name=detector,
-    template_imagefile=template_imagefile,
-    binning=(1, 1, 1),
-    preprocessing_binning=pre_binning,
-)
-
 ####################
 # Initialize setup #
 ####################
 setup = Setup(
-    beamline=beamline,
-    detector=detector,
+    beamline_name=beamline,
     energy=energy,
     rocking_angle=rocking_angle,
     distance=sdd,
@@ -258,6 +246,10 @@ setup = Setup(
     sample_offsets=sample_offsets,
     actuators=actuators,
     is_series=is_series,
+    detector_name=detector,
+    template_imagefile=template_imagefile,
+    binning=(1, 1, 1),
+    preprocessing_binning=pre_binning,
 )
 
 ########################
@@ -279,23 +271,25 @@ print(f'{"#"*(5+len(str(scan)))}\nScan {scan}\n{"#"*(5+len(str(scan)))}')
 print("\n##############\nSetup instance\n##############")
 print(setup)
 print("\n#################\nDetector instance\n#################")
-print(detector)
+print(setup.detector)
 
 ##########################
 # Initialize the logfile #
 ##########################
 if simulation:
-    detector.datadir, detector.datadir, detector.savedir = (root_folder,) * 3
+    setup.detector.datadir, setup.detector.datadir, setup.detector.savedir = (
+        root_folder,
+    ) * 3
 
 logfile = setup.create_logfile(
-    scan_number=scan, root_folder=root_folder, filename=detector.specfile
+    scan_number=scan, root_folder=root_folder, filename=setup.detector.specfile
 )
 
 #############################################
 # Initialize geometry for orthogonalization #
 #############################################
 qconv, offsets = setup.init_qconversion()
-detector.offsets = offsets
+setup.detector.offsets = offsets
 hxrd = xu.experiment.HXRD(sample_inplane, sample_outofplane, qconv=qconv)
 # x downstream, y outboard, z vertical
 # first two arguments in HXRD are the inplane reference direction
@@ -304,8 +298,7 @@ hxrd = xu.experiment.HXRD(sample_inplane, sample_outofplane, qconv=qconv)
 ###################
 # define colormap #
 ###################
-colormap = gu.Colormap()
-my_cmap = colormap.cmap
+my_cmap = ColormapFactory().cmap
 
 ###################################
 # load experimental data and mask #
@@ -314,7 +307,7 @@ plt.ion()
 root = tk.Tk()
 root.withdraw()
 file_path = filedialog.askopenfilename(
-    initialdir=detector.scandir,
+    initialdir=setup.detector.scandir,
     title="Select diffraction pattern",
     filetypes=[("NPZ", "*.npz"), ("NPY", "*.npy")],
 )
@@ -322,7 +315,7 @@ diff_pattern, _ = util.load_file(file_path)
 diff_pattern = diff_pattern.astype(float)
 
 file_path = filedialog.askopenfilename(
-    initialdir=detector.scandir,
+    initialdir=setup.detector.scandir,
     title="Select mask",
     filetypes=[("NPZ", "*.npz"), ("NPY", "*.npy")],
 )
@@ -357,11 +350,11 @@ if crop_roi is not None:
 # compensate the "rebin" option used in PyNX  #
 ###############################################
 # update also the detector pixel sizes to take into account the binning
-detector.binning = phasing_binning
+setup.detector.binning = phasing_binning
 print(
     "Pixel sizes after phasing_binning (vertical, horizontal): ",
-    detector.pixelsize_y,
-    detector.pixelsize_x,
+    setup.detector.pixelsize_y,
+    setup.detector.pixelsize_x,
     "(m)",
 )
 diff_pattern = util.bin_data(
@@ -433,8 +426,8 @@ hxrd.Ang2Q.init_area(
     cch2=int(x0),
     Nch1=numy,
     Nch2=numx,
-    pwidth1=detector.pixelsize_y,
-    pwidth2=detector.pixelsize_x,
+    pwidth1=setup.detector.pixelsize_y,
+    pwidth2=setup.detector.pixelsize_x,
     distance=setup.distance,
 )
 # first two arguments in init_area are the direction of the detector
@@ -445,7 +438,6 @@ if simulation:
     )
 else:
     qx, qz, qy, _ = setup.calc_qvalues_xrutils(
-        logfile=logfile,
         hxrd=hxrd,
         nb_frames=numz,
         scan_number=scan,
@@ -500,7 +492,7 @@ if debug:
 # load reconstructed object #
 #############################
 file_path = filedialog.askopenfilename(
-    initialdir=detector.savedir,
+    initialdir=setup.detector.savedir,
     title="Select reconstructions (prtf)",
     filetypes=[("NPZ", "*.npz"), ("NPY", "*.npy"), ("CXI", "*.cxi"), ("HDF5", "*.h5")],
 )
@@ -633,19 +625,20 @@ if flag_interact:
             vmin=0,
             reciprocal_space=True,
         )
-        np.savez_compressed(detector.savedir + "linecut_prtf.npz", data=linecut_prtf)
+        np.savez_compressed(
+            setup.detector.savedir + "linecut_prtf.npz", data=linecut_prtf
+        )
     else:
         linecut_prtf = prtf_matrix
 
     plt.ioff()
     max_colorbar = 5
+    starting_point = [z0, y0, x0]
     endpoint = [0, 0, 0]
-    cut = gu.linecut(
+    cut = lc.linecut(
         prtf_matrix,
-        start_indices=(z0, y0, x0),
-        stop_indices=endpoint,
+        indices=list(zip(starting_point, endpoint)),
         interp_order=1,
-        debugging=False,
     )
     diff_pattern[np.isnan(diff_pattern)] = 0  # discard nans
     fig_prtf, ((ax0, ax1), (ax2, ax3)) = plt.subplots(nrows=2, ncols=2, figsize=(12, 6))
@@ -756,7 +749,7 @@ ax.set_xlim(defined_q.min(), defined_q.max())
 ax.set_ylim(0, 1.1)
 
 gu.savefig(
-    savedir=detector.savedir,
+    savedir=setup.detector.savedir,
     figure=fig,
     axes=ax,
     tick_width=2,
