@@ -8,15 +8,18 @@
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
 import argparse
+import logging
 from datetime import datetime
+from pathlib import Path
+
 from matplotlib import pyplot as plt
 
 from bcdi.postprocessing.postprocessing_runner import run
-from bcdi.utils.parser import add_cli_parameters, ConfigParser
+from bcdi.utils.parser import ConfigParser, add_cli_parameters
+from bcdi.utils.snippets_logging import configure_logging
 
-CONFIG_FILE = (
-    "C:/Users/Jerome/Documents/myscripts/bcdi/bcdi/examples/config_postprocessing.yml"
-)
+here = Path(__file__).parent
+CONFIG_FILE = str(here.parents[1] / "bcdi/examples/S11_config_postprocessing.yml")
 
 helptext = """
 Interpolate the output of the phase retrieval into an orthonormal frame,
@@ -45,14 +48,16 @@ Usage:
 
     Parameters related to path names:
 
-    :param scan: e.g. 11
-     scan number
+    :param scans: e.g. 11
+     scan number or list of scan numbers
     :param root_folder: e.g. "C:/Users/Jerome/Documents/data/dataset_ID01/"
      folder of the experiment, where all scans are stored
     :param save_dir: e.g. "C:/Users/Jerome/Documents/data/dataset_ID01/test/"
-     images will be saved here, leave it to None otherwise
+     images will be saved here, leave it to None otherwise. Provide a single path or a
+     list of paths for multiple scans.
     :param data_dir: e.g. None
-     use this to override the beamline default search path for the data
+     use this to override the beamline default search path for the data. Provide
+     a list of paths for multiple scans.
     :param sample_name: e.g. "S"
      str or list of str of sample names (string in front of the scan number in the
      folder name). If only one name is indicated, it will be repeated to match the
@@ -60,16 +65,20 @@ Usage:
     :param comment: string use in filenames when saving
     :param debug: e.g. False
      True to see plots
-    :param reconstruction_file: e.g. "modes.h5"
-     path to a reconstruction file, to avoid opening a pop-up window
+    :param colormap: e.g. "turbo"
+     "turbo", "custom" or colormap defined in the colorcet package, see
+     https://colorcet.holoviz.org/
+    :param reconstruction_files: e.g. "C:/Users/Jerome/Documents/data/modes.h5"
+     full path to the output of phase retrieval, or list of such paths if 'scans' is a
+     list. Providing several reconstructions for each scan is not supported.
+     If None an interactive window will open to choose a file.
 
     Parameters used in the interactive masking GUI:
 
     :param backend: e.g. "Qt5Agg"
-     Backend used in script, change to "Agg" to make sure the figures are saved, not
-     compaticle with interactive masking. Other possibilities are
-     'module://matplotlib_inline.backend_inline'
-     default value is "Qt5Agg"
+     Backend used in script, change to "Agg" if you want to save the figures without
+     showing them. Other possibilities are "Qt5Agg" (default) and
+     "module://matplotlib_inline.backend_inline"
 
     Parameters used when averaging several reconstruction:
 
@@ -82,8 +91,11 @@ Usage:
 
     Parameters related to centering:
 
-    :param centering_method: e.g. "max_com"
-    'com' (center of mass), 'max', 'max_com' (max then com), 'do_nothing'
+    :param centering_method: e.g. {"direct_space": "max_com", "reciprocal_space": "max"}
+     dictionary with the centering methods for direct and reciprocal space. The methods
+     supported are 'com' (center of mass), 'max', 'max_com' (max along the first axis,
+     center of mass in the detector plane), 'skip'. If a simple string is provided, it
+     will use that method for both direct and reciprocal space.
     :param roll_modes: e.g. [0, 0, 0]
     correct a roll of few pixels after the decomposition into modes in PyNX
     axis=(0, 1, 2)
@@ -135,6 +147,9 @@ Usage:
     :param isosurface_strain: e.g. 0.2
      threshold use for removing the outer layer (the strain is undefined at the exact
      surface voxel)
+    :param skip_unwrap: e.g. False
+     If 'skip_unwrap', it will not unwrap the phase. It can be used when there is a
+     defect and phase unwrapping does not work well.
     :param strain_method: e.g. "default"
      how to calculate the strain, available options:
 
@@ -152,14 +167,6 @@ Usage:
      optional dictionary that can be used to define the entries corresponding to
      actuators in data files (useful at CRISTAL where the location of data keeps
      changing, or to declare a non-standard monitor)
-    :param is_series: e.g. True
-     specific to series measurement at P10
-    :param custom_scan: e.g. False
-     True for a stack of images acquired without scan, e.g. with ct in a
-     macro, or when there is no spec/log file available
-    :param custom_images: list of image numbers for the custom_scan, None otherwise
-    :param custom_monitor: list of monitor values for normalization for the custom_scan,
-     None otherwise
     :param rocking_angle: e.g. "outofplane"
      "outofplane" for a sample rotation around x outboard, "inplane" for a sample
      rotation around y vertical up
@@ -195,7 +202,8 @@ Usage:
      detector angle in deg(rotation around y vertical up, typically gamma), corrected
      for the direct beam position. Leave None to use the uncorrected position.
     :param specfile_name: e.g. "l5.spec"
-     beamline-dependent parameter, use the following template:
+     string or list of strings for multiple scans. beamline-dependent parameter,
+     use the following template:
 
      - template for ID01 and 34ID: name of the spec file if it is at the default
       location (in root_folder) or full path to the spec file
@@ -210,6 +218,9 @@ Usage:
     :param custom_scan: e.g. False
      True for a stack of images acquired without scan, e.g. with ct in a
      macro, or when there is no spec/log file available
+    :param custom_images: list of image numbers for the custom_scan, None otherwise
+    :param custom_monitor: list of monitor values for normalization for the custom_scan,
+     None otherwise
     :param custom_motors: e.g. {"delta": 5.5, "gamma": 42.2, "theta": 1.1, "phi": 0}
      dictionary providing the goniometer positions of the beamline
 
@@ -234,7 +245,8 @@ Usage:
      Leave None to use the full detector. Use with center_fft='skip' if you want this
      exact size for the output.
     :param template_imagefile: e.g. "data_mpx4_%05d.edf.gz"
-     use one of the following template:
+     string or list of strings for multiple scans. beamline-dependent parameter,
+     use the following template:
 
      - template for ID01: 'data_mpx4_%05d.edf.gz' or 'align_eiger2M_%05d.edf.gz'
      - template for SIXS_2018: 'align.spec_ascan_mu_%05d.nxs'
@@ -288,8 +300,6 @@ Usage:
 
     Parameters related to data visualization:
 
-    :param debug: e.g. False
-     True to show all plots for debugging
     :param align_axis: e.g. False
      True to rotate the crystal to align axis_to_align along ref_axis for visualization.
      This is done after the calculation of the strain and has no effect on it.
@@ -353,6 +363,11 @@ Usage:
 
 
 if __name__ == "__main__":
+
+    now = datetime.now()
+    configure_logging()
+    logging.info(f"Start script at {now}")
+
     # construct the argument parser and parse the command-line arguments
     ap = argparse.ArgumentParser()
     ap = add_cli_parameters(ap)
@@ -360,11 +375,14 @@ if __name__ == "__main__":
 
     # load the config file
     file = cli_args.get("config_file") or CONFIG_FILE
+    logging.info(f"Loading config file '{file}'")
+
     parser = ConfigParser(file, cli_args)
     args = parser.load_arguments()
-    args["time"] = f"{datetime.now()}"
+    args["time"] = f"{now}"
     run(prm=args)
 
-    print("\nEnd of script")
+    now = datetime.now()
+    logging.info(f"End of script at {now}")
     plt.ioff()
     plt.show()
