@@ -7,13 +7,9 @@
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
 import unittest
+
 import bcdi.algorithms.algorithms_utils as alg
-
-
-def run_tests(test_class):
-    suite = unittest.TestLoader().loadTestsFromTestCase(test_class)
-    runner = unittest.TextTestRunner(verbosity=2)
-    return runner.run(suite)
+from tests.config import run_tests
 
 
 class Test(unittest.TestCase):
