@@ -6,28 +6,24 @@
 #       authors:
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
-from pyfakefs import fake_filesystem_unittest
-import numpy as np
 import os
 import unittest
 from unittest.mock import patch
+
+import numpy as np
+from pyfakefs import fake_filesystem_unittest
+
 from bcdi.experiment.detector import (
-    create_detector,
-    create_roi,
     Detector,
     Dummy,
-    Merlin,
-    Timepix,
     Eiger2M,
     Eiger4M,
     Maxipix,
+    Merlin,
+    Timepix,
+    create_detector,
 )
-
-
-def run_tests(test_class):
-    suite = unittest.TestLoader().loadTestsFromTestCase(test_class)
-    runner = unittest.TextTestRunner(verbosity=2)
-    return runner.run(suite)
+from tests.config import run_tests
 
 
 class TestCreateDetector(unittest.TestCase):
@@ -68,71 +64,6 @@ class TestCreateDetector(unittest.TestCase):
             create_detector()
 
 
-class TestCreateRoi(unittest.TestCase):
-    """
-    Tests related to create_roi.
-
-    def create_roi(dic : Dict[str, Any]) -> Union[List[int], None]:
-    """
-
-    def test_not_a_dict(self):
-        with self.assertRaises(TypeError):
-            create_roi({0, 1})
-
-    def test_roi_none(self):
-        self.assertEqual(create_roi({"a": 0}), None)
-
-    def test_roi_wrong_length(self):
-        with self.assertRaises(ValueError):
-            create_roi({"roi_detector": [1, 2]})
-
-    def test_roi_wrong_type(self):
-        with self.assertRaises(TypeError):
-            create_roi({"roi_detector": "[1, 2, 3, 4]"})
-
-    def test_center_roi_x_center_roi_y_none(self):
-        roi = [10, 200, 20, 50]
-        output = create_roi({"roi_detector": roi})
-        self.assertTrue(all(out == roi[idx] for idx, out in enumerate(output)))
-
-    def test_center_roi_x_not_none(self):
-        roi = [10, 200, 20, 50]
-        correct = [10, 200, 130, 200]
-        output = create_roi({"roi_detector": roi, "center_roi_x": 150})
-        self.assertTrue(all(out == correct[idx] for idx, out in enumerate(output)))
-
-    def test_center_roi_x_wrong_type_str(self):
-        with self.assertRaises(TypeError):
-            create_roi({"roi_detector": [10, 200, 20, 50], "center_roi_x": "150"})
-
-    def test_center_roi_x_wrong_type_float(self):
-        with self.assertRaises(TypeError):
-            create_roi({"roi_detector": [10, 200, 20, 50], "center_roi_x": 1.5})
-
-    def test_center_roi_y_wrong_type_str(self):
-        with self.assertRaises(TypeError):
-            create_roi({"roi_detector": [10, 200, 20, 50], "center_roi_y": "150"})
-
-    def test_center_roi_y_wrong_type_float(self):
-        with self.assertRaises(TypeError):
-            create_roi({"roi_detector": [10, 200, 20, 50], "center_roi_y": 1.5})
-
-    def test_center_roi_y_not_none(self):
-        roi = [10, 200, 20, 50]
-        correct = [140, 350, 20, 50]
-        output = create_roi({"roi_detector": roi, "center_roi_y": 150})
-        self.assertTrue(all(out == correct[idx] for idx, out in enumerate(output)))
-
-    def test_center_roi_x_y_not_none(self):
-        roi = [10, 200, 20, 50]
-        correct = [140, 350, -10, 60]
-        output = create_roi(
-            {"roi_detector": roi, "center_roi_y": 150, "center_roi_x": 10}
-        )
-        print(output)
-        self.assertTrue(all(out == correct[idx] for idx, out in enumerate(output)))
-
-
 class TestDetector(fake_filesystem_unittest.TestCase):
     """
     Tests related to the properties of the base class.
@@ -330,8 +261,8 @@ class TestDetector(fake_filesystem_unittest.TestCase):
             Maxipix(name="Maxipix", sample_name=777)
 
     def test_sample_name_wrong_length(self):
-        with self.assertRaises(ValueError):
-            Maxipix(name="Maxipix", sample_name="")
+        det = Maxipix(name="Maxipix", sample_name="")
+        self.assertEqual(det.sample_name, "")
 
     def test_sample_name_None(self):
         det = Maxipix(name="Maxipix", sample_name=None)
@@ -412,9 +343,6 @@ class TestDetector(fake_filesystem_unittest.TestCase):
         det = Maxipix(name="Maxipix", template_imagefile="S")
         self.assertEqual(det.template_imagefile, "S")
 
-    def test_repr(self):
-        self.assertIsInstance(self.det.__repr__(), str)
-
     def test_background_subtraction_correct(self):
         data = np.ones((3, 3))
         background = np.ones((3, 3))
@@ -733,6 +661,13 @@ class TestDetector(fake_filesystem_unittest.TestCase):
         with self.assertRaises(ValueError):
             det._saturation_correction(data, mask, nb_frames=0)
 
+    def test_repr(self):
+        self.assertIsInstance(eval(repr(self.det)), Maxipix)
+
+    def test_repr_str_not_None(self):
+        self.det.template_file = "test"
+        self.assertIsInstance(eval(repr(self.det)), Maxipix)
+
 
 class TestMaxipix(unittest.TestCase):
     """Tests related to the Maxipix detector."""
