@@ -7,22 +7,24 @@
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
 import unittest
+
 from bcdi.experiment.rotation_matrix import RotationMatrix
-
-
-def run_tests(test_class):
-    suite = unittest.TestLoader().loadTestsFromTestCase(test_class)
-    runner = unittest.TextTestRunner(verbosity=2)
-    return runner.run(suite)
+from tests.config import run_tests
 
 
 class Test(unittest.TestCase):
     """Tests related to rotation matrix instantiation."""
 
+    def setUp(self) -> None:
+        self.rotmat = RotationMatrix(circle="x+", angle=34)
+
     def test_instantiation_missing_parameter(self):
         with self.assertRaises(TypeError):
             RotationMatrix()
 
+    def test_repr(self):
+        self.assertIsInstance(eval(repr(self.rotmat)), RotationMatrix)
+
 
 if __name__ == "__main__":
     run_tests(Test)
