@@ -6,15 +6,12 @@
 #       authors:
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
-import numpy as np
 import unittest
+
+import numpy as np
+
 from bcdi.experiment.setup import Setup
-
-
-def run_tests(test_class):
-    suite = unittest.TestLoader().loadTestsFromTestCase(test_class)
-    runner = unittest.TextTestRunner(verbosity=2)
-    return runner.run(suite)
+from tests.config import run_tests
 
 
 class Test(unittest.TestCase):
@@ -41,7 +38,7 @@ class TestCheckSetup(unittest.TestCase):
     """
 
     def setUp(self) -> None:
-        self.setup = Setup(beamline="ID01")
+        self.setup = Setup(beamline_name="ID01")
         self.params = {
             "grazing_angle": (1, 2),
             "inplane_angle": 1.23,
@@ -84,11 +81,6 @@ class TestCheckSetup(unittest.TestCase):
         with self.assertRaises(ValueError):
             self.setup.check_setup(**self.params)
 
-    def test_check_setup_energy_ndarray(self):
-        self.params["energy"] = np.array([10.005, 10.010, 10.015])
-        with self.assertRaises(TypeError):
-            self.setup.check_setup(**self.params)
-
     def test_check_setup_tilt_angle_predefined(self):
         self.setup.tilt_angle = 2
         self.setup.check_setup(**self.params)
@@ -115,13 +107,9 @@ class TestCheckSetup(unittest.TestCase):
         self.setup.check_setup(**self.params)
         self.assertEqual(self.setup.outofplane_angle, self.params["outofplane_angle"])
 
-    def test_check_setup_outofplane_angle_ndarray(self):
-        self.params["outofplane_angle"] = np.arange(10)
-        with self.assertRaises(TypeError):
-            self.setup.check_setup(**self.params)
-
     def test_check_setup_outofplane_angle_undefined(self):
         self.params["outofplane_angle"] = None
+        self.setup.outofplane_angle = None
         with self.assertRaises(ValueError):
             self.setup.check_setup(**self.params)
 
@@ -167,7 +155,7 @@ class TestCorrectDirectBeam(unittest.TestCase):
 
     def setUp(self) -> None:
         self.setup = Setup(
-            beamline="ID01",
+            beamline_name="ID01",
             direct_beam=[12, 324],
             dirbeam_detector_angles=[-1, 1],
             distance=1.23,
@@ -203,7 +191,7 @@ class TestCorrectDetectorAngles(unittest.TestCase):
 
     def setUp(self) -> None:
         self.setup = Setup(
-            beamline="ID01",
+            beamline_name="ID01",
             direct_beam=[12, 324],
             dirbeam_detector_angles=[0.5, 1],
             distance=1.23,
@@ -261,8 +249,23 @@ class TestCorrectDetectorAngles(unittest.TestCase):
         )
 
 
+class TestRepr(unittest.TestCase):
+    """Tests related to __repr__."""
+
+    def setUp(self) -> None:
+        self.setup = Setup(beamline_name="ID01")
+
+    def test_return_type(self):
+        self.assertIsInstance(eval(repr(self.setup)), Setup)
+
+    def test_rocking_angle_str(self):
+        self.setup.rocking_angle = "outofplane"
+        self.assertIsInstance(eval(repr(self.setup)), Setup)
+
+
 if __name__ == "__main__":
     run_tests(Test)
     run_tests(TestCheckSetup)
     run_tests(TestCorrectDirectBeam)
     run_tests(TestCorrectDetectorAngles)
+    run_tests(TestRepr)
