@@ -6,11 +6,14 @@
 #       authors:
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
+import unittest
+from pathlib import Path
+
 import numpy as np
 from pandas import DataFrame
-from pathlib import Path
-import unittest
+
 from bcdi.postprocessing.facet_analysis import Facets
+from tests.config import run_tests
 
 here = Path(__file__).parent
 THIS_DIR = str(here)
@@ -18,12 +21,6 @@ SAVEDIR = str(here.parents[1] / "test_output/")
 FILENAME = "3572_fa.vtk"
 
 
-def run_tests(test_class):
-    suite = unittest.TestLoader().loadTestsFromTestCase(test_class)
-    runner = unittest.TextTestRunner(verbosity=2)
-    return runner.run(suite)
-
-
 class TestInitFacetsParams(unittest.TestCase):
     """
     Tests on the class Facets.
@@ -225,6 +222,9 @@ class TestInitFacetsAttributes(unittest.TestCase):
     def test_init_particle_cmap(self):
         self.assertTrue(self.facets.particle_cmap == "gist_ncar")
 
+    def test_repr(self):
+        self.assertIsInstance(eval(repr(self.facets)), Facets)
+
 
 if __name__ == "__main__":
     run_tests(TestInitFacetsParams)
