@@ -6,15 +6,12 @@
 #       authors:
 #         Steven Leake, leake@esrf.fr
 
-import numpy as np
 import unittest
+
+import numpy as np
+
 import bcdi.simulation.supportMaker as sM
-
-
-def run_tests(test_class):
-    suite = unittest.TestLoader().loadTestsFromTestCase(test_class)
-    runner = unittest.TextTestRunner(verbosity=2)
-    return runner.run(suite)
+from tests.config import run_tests
 
 
 class Test(unittest.TestCase):
